#!/usr/bin/env perl
# /home/foth/dawai/bin/runtest.pl

use warnings;
use strict;
use Getopt::Std;

our $opt_d;    # dummy mode
our $opt_r;    # range of sentences
our $opt_s;    # list of sentences

getopts('dr:s:');

our ($min, $max);

if(!$opt_r && -f 'RANGE') {
  $opt_r = `cat RANGE`;
}

if($opt_r) {
  ($min,$max) = ($opt_r =~ /(\d+)-(\d+)/)
    or die "Invalid -r range!\n";
  open(OUT, ">RANGE") or die "Can't write `>RANGE': $!\n";
  print OUT $opt_r;
  close(OUT);
}

die "No binary found.\n" unless -x 'cdgp';

our $host = `uname -n`;
chomp $host;

# calculate list of available CPUs
our @hosts;

if( -r 'HOSTS' ) {
  @hosts = split(/\n/,`cat HOSTS`);
} elsif($host eq 'Siemens') {
  @hosts = qw(Siemens);
} elsif($host eq 'msi') {
  @hosts = qw(msi msi);
} else {
  @hosts = split(/\n/, `cat /home/foth/var/lib/HOSTS`);
}

# make a note of which hosts were actually used
our %done;
open(HOSTS, ">HOSTS") or die "Can't write `>HOSTS': $!\n";
for(@hosts) {
  print HOSTS "$_\n";
}
close(HOSTS);


our @commands;

chomp(our $dir = `pwd`);

our %log_exists;
opendir(DOT, ".") or die "Can't read directory . !\n";
map { $log_exists{$_}++ } readdir(DOT);
closedir DOT;

our @experiments;

# load experimental setup
do 'experiments.pl' or die "Can't read experiments.pl: $!\n";

open(IDS, ">IDS") or die "Can't write IDS: $!\n";
for my $e (@experiments) {
  if(!exists($$e{id})) {
    $$e{id} = $$e{script};
  }
  print IDS "$$e{id}\n";
}
close(IDS);

# find annotations in the grammar file
our @sentences;

if(!$opt_r) {

  @sentences = map {
    chomp;
    s/^.+:(.+)<->/$1/g or die "No match: $_";
    s/[ \']//g; $_
  } `egrep ':.+<->' *.cdg`;
}

my $base;
if( -f 'GRAMMAR') {
  $base = `cat GRAMMAR`;
  chomp $base;
} else {
  $base = $experiments[0]{grammar};
}

if($opt_s) {
  open(IN, "$opt_s") or die "Can't read `$opt_s': $!\n";
  while(<IN>) {
    chomp;
    push @sentences, $_;
  }
  close(IN);
} else {

# deduce sentence names and number from the corpus name
  if (-f 'MAX') {
    for (1.. `cat MAX` ) {
      push @sentences, "$base-s$_";
    }
  }

  if ($opt_r) {
    for ($min..$max) {
      push @sentences, "$base-s$_";
    }
  }

# try very hard to find the sentences in the annodir, if present
  if (!@sentences) {
    $base =~ s/\.cdg\n//;
    my $spec = `grep annodirs *cdg`;
    if ($spec =~ /pragma set annodirs (.*)/) {
      print "Found annodirs: $1\n";
      my $d = $1;
      my $last = `(cd $d/000 && ls -v | grep cda | tail -1)`;
      if ($last =~ /s(\d+)/) {
        for (1..$1) {
          push @sentences, "$base-s$_";
        }
      }
    }
  }
}
printf "Found %d sentences.\n", $#sentences+1;
die unless @sentences;


our %binaries;

# create N scripts containing TOTAL / N invocations each
my %current;
my $jobs = 0;
for my $s (@sentences) {
  for my $e (@experiments) {
    my $grammar = "$$e{grammar}.cdg";
    my $id = $$e{id};
    my $log = "$s.$id.log";

    my $binary = $$e{binary} || './cdgp';
    $binaries{$binary}++;
    my $extra = $$e{extra} || '';
    next if exists($log_exists{$log});
    $jobs++;
    my $thescript= "scripts/$$e{script}";
    if(! -f $thescript) {
      $thescript = "$ENV{HOME}/cdg/utils/scripts/$$e{script}";
    }
    if(! -f $thescript) {
      die "No script `$$e{script}' found.\n";
    }
    push @commands,
      "$extra CDGBINARY=$binary cdg -s $thescript $grammar $s &> $log\n";

    $current{$id}++;

  }
}
open(CURRENT, ">CURRENT") or die "Can't write CURRENT: $!\n";
for my $id (keys %current) {
  print CURRENT "$id\n";
}
close(CURRENT);


print "Scheduling $jobs parse runs.\n";

my $nrjobs = $#hosts+1;

my $i = 0;
my @lines = (map {0} @hosts);
for my $h (@hosts) {
  open(FILE, ">part$i.sh") or die "Can't write `>part$i.sh': $!\n";
  print FILE "cd $dir\n";

  # duplicate the binary to defeat memory sharing
  print FILE "cp $_ $_.$i\n" for(keys %binaries);

  my $j = 0;
  for my $c (@commands) {
    next unless $j++ % $nrjobs == $i;

    # use job-specific binary
    my $x = $c;
    $x =~ s/(CDGBINARY=[^ ]+) /$1.$i /;

    print FILE $x;
    $lines[$i]++;
  }
  print FILE "rm $_.$i\n" for(keys %binaries);
  close(FILE);
  $i++;
}

# create master script which calls ssh N times
my $script = 'runtest.sh';
open(FILE, ">$script") or die "Can't write `>$script': $!\n";

print FILE 'dir=',`pwd`;
$i=0;
for my $a (@hosts) {
  next unless $lines[$i];
  if('localhost' eq $a) {
    print FILE "sh $dir/part$i.sh&\n"; $i++;
  } else {

    my $ssh_magic = '';

    if($host !~ /^node/) {
      if($a =~ /node(\d)$/) {
        $ssh_magic = "-p 200$1";
        $a = 'nats47';
      } elsif($a =~ /node(\d\d)/) {
        $ssh_magic = "-p 20$1";
        $a = 'nats47';
      }
    }

    print FILE "echo ssh $ssh_magic $a sh $dir/part$i.sh&\n";
    print FILE "sleep 2; /usr/bin/ssh $ssh_magic $a sh $dir/part$i.sh&\n";
    $i++;
  }
}
close(FILE);
chmod 0700, $script;

# actually call the master script
system "./$script" unless $opt_d;
