#! /usr/bin/perl -w

use strict;
use locale;

#	This script validates the word references given in a CDG annotation file 
# 	and fixes them where necessary. Fixing is performed on the assumption that
#	the word numbers given in the header line of each word are indeed correct
#	and only the annotation references are corrupt. This case can arise in 
#	when changes to the sentence length have occurred while attempting to
#	maintain the same annotation file.
#
# 	Invocation parameters:
#	======================
#   1. File(s) containing the CDG2 annotations to be validated.

my $punctuation_mark = qr /[',.:;!?]/;
my $word_characters = qr /[\w']|$punctuation_mark/o;
my $label_characters = qr /[A-Z]/;
my $feature_characters = qr /[\w_]/;

# Note that these character definitions determine which characters make permissible word
# or label characters in a sentence.

# Define which levels and labels in the CDG annotation file are to be considered in the evaluation. 
# '0' means to consider only the levels specified below, '1' means to consider all levels.
	my $consider_all_levels = 1;

# If this variable is set to '1' the comparison will be done over all levels, otherwise only over the levels specified subsequently.
	my %levels_considered = (	"SYN"	=>	1,
# 								"REF"	=>	1,
# 								"AGNT"	=>	1,
# 								"THME"	=>	1, 
# 								"INST"	=>	1,
							);
					
my $consider_all_labels = 0;
# If this variable is set to '1' the comparison will be done over all labels, otherwise only over the labels specified in the hash below.

my %labels_considered = (	"AGENT"		=>	1,
							"THEME"		=>	1,
							"INSTRUMENT"	=>	1,
							"COMITATIVE" 	=>	1,
							"OWNER"		=>	1,
							"RECIPIENT"		=>	1,
						);

my @words;
my @lines;
my %word_string;
my %word_number;
my %regent_number;
my %regent_string;
my %sentence;
my %has_ROLE;
my $word_number;
my $sentence_number;
my %feature;
my %verb_forms;
my %sem_val;


while ( <> )
{		
	chomp;
	push @lines, $_;

	# Skip empty lines or lines which are known to be uninteresting from an annotation point of view.

	if ( /^$/o || /wordgraph\d/o )	
	{
		$sentence_number ++ if /wordgraph\d/o;	# Lines containing 'wordgraph' introduce a new sentence annotation.
		next;
	}
	
	
	if ( /^\s+\d+ (\d+) ($word_characters+)/o ) # $1 captures the word number in the current sentence, $2 captures the word string. 
	{	
		$word_number = $1;
		$word_string{ $sentence_number }{ $word_number } = $2;
		$word_number{ $sentence_number }{ $2 } = $1;
		
		# print "Word number for word string $2: $1 \n";
		
		push @words, $word_string{ $sentence_number }{ $word_number }; 
		
		next;
	}
	elsif ( /([A-Z]{3,4}) -> ($label_characters+) -> (\d+) \/\/ (\w+)/o ) 	# $1 captures the level, $2 captures the label and $3 captures the regent's word number, $4 captures the regent's word string. 
	{
		next unless ( ( $consider_all_levels || $levels_considered{ $1 } ) && ( $consider_all_labels || $labels_considered{ $2 } ) );  # Only hash labels to be considered.
							
		$regent_number{ $sentence_number }{ $word_number } = $3;
		$regent_string{ $sentence_number }{ $word_number } = $4;

		next;
	}
	elsif ( /^;$/o ) 
	{
		$sentence{ $sentence_number } = join " ", @words;
		
		for ( @lines )
		{
			if ( /^\s+\d+ (\d+) ($word_characters+)/o ) # $1 captures the word number in the current sentence, $2 captures the word string. 
			{	
				$word_number = $1;
			}

			if ( /[A-Z]{3,4} -> $label_characters+ -> (\d+) \/\/ (\w+)/o && $1 > 3 ) 	# $1 captures the regent's word number, $2 captures the regent's string. 
			{	
				my $string = $2;
									
				if ( "$1" ne "$word_number{ $sentence_number }{ $string }" )
				{
					s/\d+/$word_number{ $sentence_number }{ $string }/;
					# print "Replacement: \t";
				}	
			}
			print $_, "\n";
		}	
		@words = ();
		@lines = ();	
	}
}
