package cia_components.reasoning;

import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLIndividual;
import edu.stanford.smi.protegex.owl.model.OWLObjectProperty;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.OWLClass;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.inference.protegeowl.ReasonerManager;
import edu.stanford.smi.protegex.owl.inference.protegeowl.ProtegeOWLReasoner;
import edu.stanford.smi.protegex.owl.inference.protegeowl.log.ReasonerLogger;
import edu.stanford.smi.protegex.owl.inference.protegeowl.log.ReasonerLoggerListener;
import edu.stanford.smi.protegex.owl.inference.protegeowl.log.ReasonerLogRecord;
import edu.stanford.smi.protegex.owl.inference.protegeowl.log.MessageLogRecord;
import edu.stanford.smi.protegex.owl.inference.dig.reasoner.DIGReasonerIdentity;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLIndividual;
import edu.stanford.smi.protegex.owl.model.triplestore.*;
import edu.stanford.smi.protegex.owl.ui.actions.OWLProperties;
import edu.stanford.smi.protegex.owl.inference.protegeowl.task.*;
import java.util.regex.*;

import cia_components.ppc.Sentence;
import cia_components.ppc.Word;

import java.util.*;


public class DIG_Query_Manager
{   
	// Start Attribute Definitions =========================================================
	
	Connector connector;
	ProtegeOWLReasoner reasoner;
	OWLModel owl_model;
	Sentence sentence;
	HashMap context_relations, concept_to_lexicalisation, lexicalisation_to_concept;    
	Vector thematic_roles;
	String prefix, lexicalisation_relation;
	
	// End Attribute Definitions ===========================================================
    
    
    // Start Constructor Definitions =======================================================

    public DIG_Query_Manager( Connector connect, Sentence sen )
	{
		set_Connector( connect );
		set_Reasoner();
		set_OWL_Model();
		set_Sentence( sen );
		set_Prefix();
		
		classify_Taxonomy();

		set_Context_Relations();
		set_Lexicalisation_Relation();
		set_Thematic_Roles();
		
		// set_Instantiated_Concepts();
		
		// set_Concept_Lexicalisations();
		
		// System.out.println( "There are " + relations.values().size() + " values in the relations hashmap." );
		
				
		// Now get the inferred subclasses of VegetarianPizza
				
		/*	

		for (int i = 1; i<= sentence.length; i++)
		{
			is_in_Context( sentence.get_Word_Object( i ) );
		}		
	
		for( Iterator it = inferred_subclasses.iterator(); it.hasNext(); ) 
		{
			// OWLNamedClass curClass = ( OWLNamedClass ) it.next();
			// System.out.println( curClass.getName() );
		}		
		// We can also classify the whole ontology, which will put the inferred class hierarchy information 
		// directly into the Protege-OWL model.
		// System.out.println( "Classifying taxonomy." );
		// reasoner.classifyTaxonomy( null );
		// System.out.println( "Classified taxonomy." );
		// We can then use the methods on OWLNamedClass for getting inferred information, without having to 
		// make separate queries to the external DIG reasoner, as this information is now in the OWLModel.
		
		*/	
	}	
	
	// End Constructor Definitions =========================================================
		
		
	// Start Method Definitions ============================================================
	
	
	private void set_Connector( Connector con )
	{
		connector = con;
	}
	
	
	private void set_Reasoner()
	{
		reasoner = connector.get_Reasoner();
	}
		
		
	public void set_OWL_Model()
	{
		owl_model = connector.get_OWL_Model();
	}
		
		
	public OWLModel get_OWL_Model()
	{
		return owl_model;
	}

		
	private void set_Sentence( Sentence sen )
	{
		sentence = sen;
	}
	
	
	private void set_Prefix()
	{
		/*
			This method sets the value for the namespace prefix of
			the upper ontology. With this prefix it is possible to 
			reference entities in the context model unambiguosly 
			even if the context model is composed of a fusion of
			several ontologies such as an upper and a lower onotology.
		*/
		
		if ( System.getenv( "UPPER_ONTOLOGY_PREFIX" ) == null )
		{
			prefix = "";
		}
		else
		{
			prefix = System.getenv( "UPPER_ONTOLOGY_PREFIX" );
		}
		
		// System.out.println( "Upper Ontology Prefix: >" + prefix + "<." );
	}

		
	public Collection <OWLNamedClass> get_Asserted_Subclasses( String class_name )
	{	
		/*
			Returns all classes located directly under the class 'class_name'
			in the OWL Model. This collection does NOT include inferred subclasses.
		*/
				
		Collection <OWLNamedClass> result = null;
		try
		{
			OWLNamedClass owl_named_class = get_Named_Class( class_name );
			result = owl_named_class.getNamedSubclasses();
		}
		catch( Exception e )
		{			
			System.err.println( "The DIG_Query_Manager could not determine the asserted subclasses for class '" + class_name + "' because: \n" + e + "\n" );
		}
		return result;
	}
	
	
	public Collection get_Asserted_Superclasses( String class_name )
	{	
		/*
			Status: WORKS OK - 19.09.08
			
			Returns all parent classes located directly above the class 'class_name'
			in the OWL Model in the asserted hierarchy, i.e.: before the inference process. 
			This collection therefore does NOT include the inferred superclasses.
		*/
		
		Collection result = null;
		try
		{
			OWLNamedClass owl_class = get_Named_Class( class_name );
			result = owl_class.getNamedSuperclasses();
		}
		catch( Exception e )
		{
			System.err.println( e );
		}
		return result;
	}
	
	
	public int get_Number_of_Asserted_Superclasses( String class_name )
	{	
		/*
			Status: WORKS OK - 12.05.07
		*/
		
		int result = 0;
		
		try
		{
			result = get_Asserted_Superclasses( class_name ).size();
		}
		catch ( Exception e )
		{
		}
			
		return result;
	}
	
	
	public boolean has_Thematically_Relevant_Instance( String input )
	{	
		/*
			Status: 
			
			Checks if there is a concept by the name of word string 'input'
			that has a thematically relevant instance in the Context Model
			(i. e.: if there is an instance of the concept which attaches 
			to a verb instance via a thematic role relation).
		*/
		
		boolean result = get_Thematically_Relevant_Individual_Names().contains( input );
		
		return result;
	}


	public OWLNamedClass get_Named_Class( String class_name )
	{	
		/*
			This method returns an OWLNamedClass object by the name of 'class_name'.
			Essentially, this method bridges the gap between the name string of a 
			class and the OWL object associated with that class name in the implementation.
			For queries against the upper ontology the appropriate namespace
			prefix is added. 
		*/
				
		OWLNamedClass result;
		
		result = owl_model.getOWLNamedClass( prefix + class_name );
			
		return result;
	}
	
	
	public RDFProperty get_RDF_Property( String property_name )
	{	
		/*
			This method returns an RDFProperty object by the name of 'property_name'.
			Essentially, this method bridges the gap between the name string of a 
			property and the RDF object associated with that property name in the implementation. 
		*/
		
		RDFProperty result;
		result = owl_model.getRDFProperty( prefix + property_name );
		
		return result;
	}


	/**
	This method returns an RDFIndividual object by the name of 'individual_name'.
	Essentially, this method bridges the gap between the name string of an 
	individual and the RDF object associated with that property name in the implementation. 
	 */
	public RDFResource get_RDF_Resource( String individual_name )
	{	
		RDFResource result = owl_model.getRDFIndividual ( prefix + individual_name );
		
		return result;
	}


	public Collection get_All_Restrictions_for( String class_name )
	{	
		/*
			Status: WORKS OK - 19.09.08
			
			This method first retrieves the OWLNamedClass object by the name of 'class_name' from the OWL Model
			and then returns all restrictions defined for this class including inherited, necessary and 
			sufficient properties. Since many of the restrictions are defined in terms of Protege-internal 
			references to other classes, the output of this method is somewhat cryptic:
			DefaultOWLHasValue(@_:A48, FrameID(5:10184 1))
		*/
		
		Collection result = new Vector();
		OWLNamedClass owl_named_class = get_Named_Class( class_name );
		
		result = owl_named_class.getRestrictions( true ); 	
		
		return result;
	}

	
	public Collection get_Direct_Restrictions_for( String class_name )
	{	
		/*
			Status: WORKS OK - 19.09.08
			
			This method first retrieves the OWLNamedClass object by the name of 'class_name' from the OWL Model
			and then returns all direct restrictions defined for this class. Restrictions inherited from
			superclasses will not be returned.
			Since many of the restrictions are defined in terms of Protege-internal 
			references to other classes, the output of this method is somewhat cryptic:
			DefaultOWLHasValue(@_:A48, FrameID(5:10184 1))
		*/
				
		Collection result = new Vector();
		
		OWLNamedClass owl_named_class = get_Named_Class( class_name );
		
		try
		{
			result = owl_named_class.getRestrictions( false ); 	
			System.out.println( "Found " + result.size() + " restrictions." );
		}
		catch( Exception e )
		{
			// System.out.println( "Found no restrictions for class '" + class_name + "'." );
		}
		
		return result;
	}

	
	/**
	*	Status: WORKS OK - 19.09.08
	*
	*	Tells whether the OWLNamedClass by the name of the input string 'class_name' 
	*	has a direct superclass in the asserted hierarchy or not.
	*
	*	It is unclear whether this method only refers to superclasses from the 
	*	asserted hierarchy or the inferred hierarchy as well.
	*/

	public boolean has_Superclass( String class_name )
	{	
		boolean result = false;
		
		// Generate an OWLNamedClass object by the name of the input string 'class_name'.
		OWLNamedClass owl_named_class = get_Named_Class( class_name );
		
		try
		{
			result = owl_named_class.hasNamedSuperclass();		
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine whether the class " + class_name + " has a superclass because:\n" + e + "\n" );
		}
		
		return result;
	}
		
	
	public void classify_Taxonomy()
	{
		/*
			This method classifies the ontology and makes the inferred information explicit in the OWL Model.
			As a result, the inferred classes and classifications can directly be obtained from the OWL Model.
			They therefore do not need to be retrieved via the external DIG reasoner anymore once this method
			has been invoked.
		*/
		
		try
		{
			reasoner.classifyTaxonomy( null );
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager (line 250) could not classify the context model because:\n" + e + "\n" );
		}
	}
	
		
	public int get_Number_of_Asserted_Subclasses( String class_name )
	{	
		/*
			Status: WORKS OK - 14.05.07
			
			This method returns the number of subclasses defined for a class 
			by the name 'class_name' in the asserted hierarchy.
		*/
		
		int result = 0;
		
		try
		{
			get_Asserted_Subclasses( class_name ).size();
		}
		catch( Exception e )
		{
		}
		
		return result;
	}
	
	
	public Collection <RDFSClass> get_Inferred_Subclasses( String class_name ) 
	{	
		/*
			Status: WORKS OK - 14.05.07
			
			This method returns the collection of all immediate subclasses of the class 
			by the name 'class_name' in the inferred hierarchy.
		*/
		
		Collection <RDFSClass> result = new Vector <RDFSClass> ();
		OWLNamedClass owl_named_class;
		
		try
		{
			owl_named_class = get_Named_Class( class_name );
			result = owl_named_class.getInferredSubclasses();
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not retrieve the inferred subclasses of " + class_name + " because: \n" + e + "\n" );
		}
		
		return result;
	}
		
		
	public int get_Number_of_Inferred_Subclasses( String class_name ) 
	{	
		/*
			Status: WORKS OK - 14.05.07
		
			This method returns the number of immediate subclasses of the class 
			'class_name' in the infererred hierarchy.
		*/
			
		int result;
		
		try
		{
			result = get_Inferred_Subclasses( class_name ).size();
		}
		catch( Exception e )			
		{
			result = 0;
		}
		
		return result;
	}
	
	
	public Collection get_Asserted_OWL_Classes() 
	{	
		/*
			Status: WORKS OK - 09.10.07
		
			This method returns a collection of all classes as defined 
			in the context model's asserted hierarchy. The collection 
			therefore contains objects of type OWLNamedClass.
		*/
		
		Collection result = owl_model.getUserDefinedOWLNamedClasses();
		
		return result;
	}
	
	
	public int get_Number_of_Asserted_OWL_Classes() 
	{	
		/*
			Status: WORKS OK - 19.09.08
		
			This method returns the number of all classes as defined 
			in the context model's asserted hierarchy.
		*/
		
		int result; 
		Collection col = owl_model.getUserDefinedOWLNamedClasses();
		result = col.size();
		
		return result;
	}
	
	
	public Collection get_Asserted_OWL_Class_Names() 
	{	
		/*
			Status: WORKS OK - 19.09.08
		
			This method returns the collection of all class names for the OWL classes defined in the context model's asserted hierarchy.
		*/
		
		Collection col = get_Asserted_OWL_Classes();
		// System.out.println( "Number of OWLClasses: " + col.size() );
		Iterator iter = col.iterator();
		Pattern pattern = Pattern.compile( "^DefaultOWLNamedClass\\((.*),\\s" ); 
		Matcher matcher = null;
		String element_name = "";
		Vector result = new Vector();
		int counter = 0;
		
		while ( iter.hasNext() )
		{
			counter ++;
			element_name = iter.next().toString().trim();	
			// System.out.println( "Element no. " + counter + " prior to modification is: " + element_name );	
			matcher = pattern.matcher( element_name );
			// System.out.println( "Matcher: " + matcher + " matched: " + matcher.find() );
							
			if ( matcher.find() )					
			{
				element_name = matcher.group( 1 );
				// System.out.println( "Element no. " + counter + " after to modification is: " + element_name + "\n" );
				result.add( element_name );
			}		
		}
		
		return result;
	}

	
	public Collection get_Asserted_OWLRelation_Names() 
	{	
		/*
			Status: WORKS OK - 19.09.08
		
			This method returns the collection of all relation names available 
			in the asserted hierarchy of the OWL model.
		*/
		
		Collection col = get_Asserted_OWL_Relations();
		// System.out.println( "Number of OWL_Relations: " + col.size() );
		Iterator iter = col.iterator();
		Pattern pattern = Pattern.compile( "^DefaultOWLObjectProperty\\((.*)\\)" );
		Matcher matcher = null;
		String element_name = "";
		Vector result = new Vector();
		int counter = 0;
		
		while ( iter.hasNext() )
		{
			counter ++;
			element_name = iter.next().toString().trim();	
			// System.out.println( "Element no. " + counter + " prior to modification is: " + element_name );	
			matcher = pattern.matcher( element_name );
			// System.out.println( "Matcher: " + matcher + " matched: " + matcher.find() );
							
			if ( matcher.find() )					
			{
				element_name = matcher.group( 1 );
				// System.out.println( "Element no. " + counter + " after to modification is: " + element_name + "\n" );
				result.add( element_name );
			}		
		}
		
		return result;
	}

	
	public Collection get_Asserted_Individual_Names() 
	{	
		/*
			Status: WORKS OK - 09.10.07
		
			This method returns the collection of all individual names in the OWL classes defined in the context model's asserted hierarchy.
		*/
		
		// System.out.println( "Number of asserted individuals: " + col.size() );
		Iterator iter = get_Asserted_Individuals().iterator();
		Pattern pattern = Pattern.compile( "^SimpleInstance\\((.*) of \\[" );
		Matcher matcher = null;
		String element_name = "";
		Vector result = new Vector();
		
		while ( iter.hasNext() )
		{
			element_name = iter.next().toString().trim();	
			// System.out.println( "Element no. " + counter + " prior to modification is: " + element_name );	
			matcher = pattern.matcher( element_name );
			// System.out.println( "Matcher: " + matcher + " matched: " + matcher.find() );
							
			if ( matcher.find() )					
			{
				element_name = matcher.group( 1 );
				// System.out.println( "Element after to modification is: " + element_name + "\n" );
				result.add( element_name );
			}		
		}
		
		return result;
	}
	
	
	public Collection get_All_Inferred_Thematically_Relevant_Individuals() 
	{	
		/*
			Status: 
		
			This method returns the collection of all individuals in the 
			inferred hierarchy which are the object of a thematically relevant 
			relation such as has_AGENT.
		*/
		
		Vector result = new Vector();
			
		try
		{
			result.add( get_Inferred_Individuals( prefix + "Event" ) );				
			result.add( get_Inferred_Individuals( prefix + "Participant" ) );				
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the collection of all thematically relevant individuals because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	/**
	 *		Status: 
	 *
	 *		This method returns the collection of all individuals in 
	 *		the inferred hierarchy which have been classified as belonging
	 *		to the class 'role'.
	 */
	
	public Collection <OWLIndividual> get_All_Inferred_Rolefillers( String role )
	{			
		Collection <OWLIndividual> result = new Vector <OWLIndividual> ();
			
		try
		{
			result.addAll( get_Inferred_Individuals( role ) );				
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the collection of " + role + " individuals because: \n" + e + "\n" );
		}
		
		return result;
	}

	
	/**
	*		Status: Works OK - 02.10.08
	*	
	*		This method returns a collection of OWLIndividuals from the inferred 
	*		hierarchy which have an is_ROLE_for relation with the event 'event_name'.
	*/
	
	public Collection <OWLIndividual>  get_Inferred_Rolefillers_for( String role, String event_name )
	{	
		Collection <OWLIndividual> result = new Vector <OWLIndividual> ();

		RDFResource event = get_RDF_Resource( event_name );
			
		try
		{		
			for ( OWLIndividual subject : get_All_Inferred_Rolefillers( role ) )
			{					
				if ( list_Objects( subject.getBrowserText(), prefix + "is_" + role + "_for" ).contains( event ) )
 				{
 					result.add( subject );				
 				}
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.err.println( "The DIG_Query_Manager could not determine the " + role + "  individuals for event " + event_name +  " because: \n" + e.getMessage() + "\n" );
		}
		
		return result;
	}

	
	public Collection get_Thematically_Relevant_Individual_Names() 
	{	
		/*
			Status: WORKS OK - 10.10.07
		
			This method returns the collection of names of all individuals 
			in the Context Model which participate in a thematic verb-entity 
			relation.
		*/
		
		Iterator iter = get_All_Inferred_Thematically_Relevant_Individuals().iterator();
		Pattern pattern = Pattern.compile( "^SimpleInstance\\((.*_\\d+) of \\[" );
		Matcher matcher = null;
		String element_name = "";
		Vector result = new Vector();
		
		try
		{
			while ( iter.hasNext() )
			{
				element_name = iter.next().toString().trim();	
				// System.out.println( "Element no. " + counter + " prior to modification is: " + element_name );	
				matcher = pattern.matcher( element_name );
				// System.out.println( "Matcher: " + matcher + " matched: " + matcher.find() );
										
				if ( matcher.find() )					
				{
					element_name = matcher.group( 1 );
					result.add( element_name );
				}		
			}
		}
		catch( Exception e )
		{
		}		
		
		return result;
	}


	public Collection get_Asserted_OWL_Relations_between( String subject, String object ) 
	{	
		/*
			Status: OK - 23.09.08
			
			This method returns a collection of all RDFProperty relations that hold 
			between the two input entities subject and object in the OWL Model's
			asserted hierarchy, including thematic and non-thematic relations.
		*/
		
		Collection result = new Vector(); 
		
		// Obtain all subject-relation references to the given object.
		Iterator iter = list_All_References_to( object );		
			
		if ( iter != null )
		{
			// Now check which of these references contain the given subject as subject.
			
			while ( iter.hasNext() )
			{
				Tuple tuple = (Tuple ) iter.next();
				String subject_candidate = tuple.getSubject().getBrowserText();
				
				// System.out.println( "Subject candidate: " + subject_candidate );
				
				if ( subject_candidate.equals( subject ) )
				{
					RDFProperty relation = tuple.getPredicate();
					result.add( relation );
					// System.out.println( "Subject '" + subject + "'----Relation '" + relation.getBrowserText() + "'----> Object '" + object + "'" );
				}
			}
		}
		
		return result;
	}
	
	
	public String get_Thematic_Relation_between( String subject, String object ) 
	{	
		/*
			Status: OK - 25.09.08
			
			This method returns the name of the thematic relation that holds 
			between the two input entities 'subject' and 'object' in the OWL 
			Model's asserted hierarchy.
		*/
		
		String result = "none"; 
		Collection thematic_relations = get_Thematic_OWL_Relations(); 
		
		if ( thematic_relations.size() > 0 )
		{
			RDFProperty relation; 
			
			for ( Object relation_object : get_Asserted_OWL_Relations_between( subject, object ) )	// Iterate over all relations between the two entities.
			{
				relation = (RDFProperty) relation_object;
				
				// System.out.println( "There are " + get_Asserted_OWL_Relations_between( subject, object ).size() + " OWL relations between subject '" + subject + "' and object '" + object + "'." );
					
				if ( thematic_relations.contains( relation ) )					
				{
					result = relation.getBrowserText();
					return result;
					// System.out.println( "There is the thematic relation " + relation.getBrowserText() + " between subject '" + subject + "' and object '" + object + "'." );
				}
			}
		}
				
		return result;
	}
	
	
	public boolean is_Thematically_Related( String subject, String object ) 
	{	
		/*
			Status: OK - 23.09.08
			
			This method checks if the two input entities 'subject'
			and 'object' are related via a thematical relation in 
			the OWL Model's asserted hierarchy.
		*/
		
		boolean result = false;
				
		if ( ! get_Thematic_Relation_between( subject, object ).equals( "none" ) ) 
		{
			result = true;
		}
		
		return result;
	}
	
	
	public Vector get_Thematic_OWL_Relations() 
	{	
		/*
			Status: Works OK - 23.09.08
			
			This method returns a vector of all RDFPropertieswhose name
			matches the pattern is_ROLE_for where ROLE is one of the thematic
			roles listed in the environment variable $SUPPORTED_THEMATIC_ROLES as 
			defined in the configuration script ~/cdg2/PPC/configure. The data
			structure 'vector' has been chosen to have control over the sequence
			in which the elements are listed.
		*/
		
		Vector result = new Vector(); 
		
		for ( Object relation : get_Asserted_OWL_Relations() )	// Iterate over all relations in the OWL Model.
		{		
			// Now check which of these relations match the 'pattern is_ROLE_for'.

			String relation_name = ( (RDFProperty) relation ).getBrowserText();
			
			// Obtain all thematic roles supported in the current set-up.
			
			for ( String thematic_role : System.getenv( "SUPPORTED_THEMATIC_ROLES" ).split( " " ) )	// Iterate over all thematic roles in the implementation.
			{			
				Pattern pattern = Pattern.compile( "is_" + thematic_role + "_for" );
				Matcher matcher = pattern.matcher( relation_name );
									
				if ( matcher.find() )					
				{
					result.add( relation );
				}
			}
		}
		
		// Note that this is a vector of RDFProperty objects.
		return result;	
	}


	public Vector get_Thematic_OWL_Relation_Names() 
	{	
		/*
			Status: Works OK - 24.09.08
			
			This method returns the collection of strings containing the names 
			of all thematic role relations found in the OWL Model.
		*/
				
		Vector result = new Vector();
		
		for ( Object relation : get_Thematic_OWL_Relations() )	// Iterate over all thematic relation objects in the OWL Model.
		{		
			String relation_name = ( (RDFProperty) relation ).getBrowserText();
			
			result.add( relation_name );
		}
		return result;
	}
	
	
	/**
	* 	Status: 
	*
	*	This method sets the vector of strings containing the names 
	*	of the thematic roles supported by the implementation. The 
	*	data structure vector was chosen to have control over the 
	*	sequence in which these strings are listed.
	*/
	
	private void set_Thematic_Roles() 
	{				
		thematic_roles = new Vector <String> ();
		
		for ( String thematic_role : System.getenv( "SUPPORTED_THEMATIC_ROLES" ).split( " " ) )	// Iterate over all thematic roles in the implementation.
		{	
			thematic_roles.add( thematic_role );
		}
	}
	
		
	public Vector <String> get_Thematic_Roles() 
	{	
		return thematic_roles;
	}
	
	
	private void set_Lexicalisation_Relation() 
	{	
		/*
			This method sets the name for the lexicalisation relation
			mapping concepts onto their lexicalisation as used in the 
			ontology to the value provided in the configuration
			script ~/cdg2/PPC/configure.
		*/
		
		lexicalisation_relation = System.getenv( "LEXICALISATION_RELATION" );
	}
	
	
	public Collection get_Context_Events( ) 
	{	
		/*
			Status: 
			
			This method returns a collection of all RDFResources that .
		*/
		
		Collection result = new Vector(); 

// 		// Obtain all subject-relation references to the given object.
// 		Iterator iter = list_All_References_to( object );		
// 		
// 		// Now check which of these references contain the given subject as subject.
// 		
// 		while ( iter.hasNext() )
// 		{
// 			Tuple tuple = (Tuple ) iter.next();
// 			String subject_candidate = tuple.getSubject().getBrowserText();
// 			
// 			System.out.println( "Subject candidate: " + subject_candidate );
// 			
// 			if ( subject_candidate.equals( subject ) )
// 			{
// 				RDFProperty relation = tuple.getPredicate();
// 				result.add( relation );
// 				System.out.println( "Subject '" + subject + "'----Relation '" + relation.getBrowserText() + "'----> Object '" + object + "'" );
// 			}
// 		}
// 		
		return result;
	}

	
	public Collection <RDFProperty> get_Asserted_OWL_Relations()
	{
	   	Collection <RDFProperty> result = owl_model.getUserDefinedOWLProperties();
		return result;
	}
	
	
/*	
	public Collection get_Connecting_Verb_Instances( String concept_1, String concept_2 )
	{
	   	/*
			This method returns the collection of verb concept instances which connect instances of concept_1 and concept_2.
		/

	   	// Collection result = ;
		return result;
	}
	
	
	public Collection get_Thematically_Attached_Noun_Instances( String verb_concept_instance )
	{
	   	/*
			This method returns the collection of noun concept instance names that connect to the given verb_concept instance.
			Think about whether it would be useful to also include the thematic role name via which the verb concept instances 
			attach to the noun concept instances.
		/

	   	Collection result = null;
		return result;
	}



	public Collection get_Thematically_Attached_Verb_Concepts( String noun_concept )
	{
	   	/*
			This method returns the collection of verb concept names (e. g. 'etw_kaufen') to which instances of the given noun_concept attach.
			Think about whether it would be useful to also include the thematic role name via which the noun instances 
			attach to the verb concept instances.
		/

	   	Collection result = null;
		return result;
	}


	public Collection get_Thematically_Attached_Verb_Instances( String noun_concept )
	{
	   	/*
			This method returns the collection of verb concept instance names (e. g. 'kaufen_1') to which instances of the given noun_concept attach.
			Think about whether it would be useful to also include the thematic role name via which the noun instances 
			attach to the verb concept instances.
		/

	   	// Collection result = ;
		return result;
	}
/*

	/*
	public Collection get_Instances_with_Relation_Name( String relation_name )
	{
     		The maximum number of matching relations has been arbitrarily set to 100.
	   	Collection result = owl_model.getMatchingResources( RDFProperty property, String relation_name, 100 );
		return result;
	}
	*/
	
	
	public Collection get_Inferred_Superclasses( String class_name ) 
	{	
		/*
			Status: WORKS OK - 14.05.07
		
			This method returns the direct superclasses of the class 'class_name'
			as determiend by the inferred hierarchy.
		*/
		
		Collection result = null;
		OWLNamedClass owl_named_class;
		
		try
		{
			owl_named_class = get_Named_Class( class_name );
			result = owl_named_class.getInferredSuperclasses();
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.err.println( "The DIG_Query_Manager could not retrieve the inferred superclasses for the class " + class_name + " because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	public String get_Reasoner_Name()
	{	
		String reasoner_name = reasoner.getIdentity().getName();
		
		return reasoner_name;
	}
	
	
	public String get_Reasoner_Version()
	{	
		String reasoner_version = reasoner.getIdentity().getVersion();

		return reasoner_version;
	}
	
	
	public Collection get_Reasoner_TELL_Elements()
	{	
		Collection reasoner_tell_elements = reasoner.getIdentity().getSupportedTellElements();

		return reasoner_tell_elements;
	}
	
	
	public Collection get_OWLRestrictions_for( String property )
	{	
		Collection result = owl_model.getOWLRestrictionsOnProperty( get_RDF_Property( property ) );
		return result;
	}
	    
		
	public Collection get_Reasoner_ASK_Elements()
	{	
		Collection reasoner_ask_elements = reasoner.getIdentity().getSupportedAskElements();

		return reasoner_ask_elements;
	}
	
	
	public Sentence get_Sentence()
	{
		return sentence;
	}
		
		
	public Collection get_Asserted_Individuals_of( String class_name )
	{
		/*
			This method returns the collection of individuals in the class 
			'class_name' as listed in the OWL model's asserted hierarchy.
		*/
		Collection result = null;
		
		try
		{
			GetIndividualsBelongingToConceptTask task = new GetIndividualsBelongingToConceptTask( get_Named_Class( class_name ), reasoner );
			task.run();
			result = task.getResult();
		}
		catch( Exception e )
		{
			System.err.println( "DIG_Query_Manager, Line 905: The DIG_Query_Manager could not retrieve the individuals in class '" + class_name + "' because: \n" + e + "\n" );
		}
		
		return result;
	}
			
	
	public Collection <OWLIndividual> get_Inferred_Individuals( String class_name )
	{
		/*
			This method returns the collection of OWLIndividuals directly inferred for the class by the name of 'class_name' 
			in the inferred hierarchy. Note that this method only returns the individuals directly contained in the given 
			class and does not provide any individuals contained in subclasses of the given class.
			The output is a collection of OWLIndividuals SimpleInstance(Instance_Name of [Cls(Class_Name, FrameID(5:10151 1))])
			where 'Instance_Name' is the name that the instance has been given and 'Class_Name' is the name of the 
			class it is instantiating. 
		*/
		
		Collection result = new Vector();
		
		OWLNamedClass owl_named_class = get_Named_Class( class_name );
		
		try
		{	
			GetIndividualsBelongingToConceptTask task = new GetIndividualsBelongingToConceptTask( get_Named_Class( class_name ), reasoner );
			task.run();
			result = task.getResult();
			// result = reasoner.getIndividualsBelongingToClass( owl_named_class, null );
			// result = owl_model.getOWLIndividuals( owl_named_class, null );
			/*
				The 'null' parameter indicates that the method is called without a 
				task listener instance since the progress of the task does not need 
				to be monitored.
			*/
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.err.println( "The DIG_Query_Manager could not retrieve the collection of inferred OWLIndividuals for the class " + class_name + " because: \n" + e + "\n" );
		}
		
		return result;
	}		
		
	
	public Collection get_Inferred_Instantiated_Class_Names( String indiv )
	{
		/*
			This method returns a collection of class names for which the individual
			by the name of 'indiv' is an instance of in the inferred hierarchy. Note
			that this collection also includes the anonymous inferred classes, i.e.
			the classes wich do not have a name but are formed as the result of a
			relation pointing towards the individual (e.g. the class of all entities
			which relate to 'indiv' via an 'is_AGENT_for' relation).
		*/
		
		Collection result = null;
		
		try
		{
			OWLIndividual individual = owl_model.getOWLIndividual( indiv );
			Collection classes = reasoner.getIndividualTypes( individual, null );
			/*
				The 'null' parameter indicates that the method is called without a 
				task listener instance since the progress of the task does not need 
				to be monitored.
			*/
			
// 			try
// 			{
// 				
// 				Pattern pattern = Pattern.compile( "is_" + thematic_role + "_for" );
// 
// 				for ( String thematic_role : System.getenv( "SUPPORTED_THEMATIC_ROLES" ).split( " " ) )	// Iterate over all thematic roles in the implementation.
// 				{			
// 					Matcher matcher = pattern.matcher( relation_name );
// 										
// 					if ( matcher.find() )					
// 					{
// 						relations.add( thematic_role );
// 						// System.out.println( "Detected semantic relation '" + relation_name + "' for object '" + object + "'." );
// 					}
// 				}
// 
// 				for ( Object type_object : classes ) 
// 				{
// 					System.out.println( "The iterator contained a class name." );
// 					String class_name = (String) type_object;
// 					result.add( class_name );
// 				}
// 			}
// 			catch( Exception e )
// 			{
// 				System.out.println( "There were no names of inferred instantiated classes in the iterator." );
// 			}
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine which inferred classes were instantiated by the individual '" + indiv + "' because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	
	public boolean is_Subsumed_by( String class_name1, String class_name2 )
	{
		/*
			Status: 19.09.08 - Seems to work with inverted argument structure!
			
			This method checks if the class by the name of 'class_name1' is
			subsumed by the class by the name of 'class_name2' in the 
			asserted hierarchy.
		*/
		
		boolean result = false;
		
		try
		{
			OWLNamedClass subsumee = get_Named_Class( class_name1 );
			OWLNamedClass subsumer = get_Named_Class( class_name2 );
			
			result = reasoner.isSubsumedBy( subsumee, subsumer, null );	
			// The 'null' indicates that the method is called without a task listener
			// instance since the progress of the task does not need to be monitored.
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine whether class '" + class_name1 + "' is subsumed by class '" + class_name2 + "' because: \n" + e + "\n" );
		}
		
		return result;
	}


	
	public Collection get_OWL_Properties()
	{
		/*
			Status: 
			This method returns the OWL properties or relations defined by the user
			in the asserted hierarchy of the OWL model.
		*/
		
		Collection result = null;		
		
		try
		{
			result = owl_model.getUserDefinedOWLProperties();
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the OWL properties defined for the OWL model '" + System.getenv( "CONTEXT_MODEL" ) + "' because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	
	public Collection <OWLIndividual> get_Asserted_Individuals()
	{
		Collection <OWLIndividual> result = new Vector <OWLIndividual> ();		
		
		try
		{
			result = owl_model.getRDFIndividuals();
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the RDF instances in the OWL model because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	
	public Collection get_OWL_Restrictions( String property )
	{
		/*
			Status: 
		*/
		
		Collection result = null;		
		
		try
		{
			result = owl_model.getOWLRestrictionsOnProperty( get_RDF_Property( property ) );
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the restrictions on property '" + property + "' because: \n" + e + "\n" );
		}
		
		return result;
	}
	
	
	
	public boolean is_Valid_Property_Value( String subject, String property, String object )
	{
		/*
			Status: OK - 23.09.08
			
			This method checks if the given subject can in principle be related to the given 
			object via the given property. Note that it does not check whether the relation
			is actually asserted in the OWL model! It only states whether such a relation
			would be feasible based on data types.
		*/
		
		boolean result = false;		
		
		try
		{
			result = owl_model.isValidPropertyValue( get_RDF_Resource( subject ), get_RDF_Property( property ), get_RDF_Resource( object ) );
		}
		catch( Exception e )
		{
			// System.err.println( "The DIG_Query_Manager could not determine whether '" + object + "' is a valid value for resource '" + subject + "''s property '" + property + "' because: \n" + e + "\n" );
		}
		
		return result;
	}

	
	
	public String get_Semantic_Valence( String object )
	{
		/*
			Status: 
			
			This method returns the semantic valence of an entity 
			based on the thematic relations it participates in as
			an object in the OWL model.
		*/
		
		String result = "";		
		
		Collection relations = new Vector();
		
		Iterator iter = list_All_References_to( object ); 
		
		if ( iter != null )
		{
			while ( iter.hasNext() )
			{
				Tuple Tuple = (Tuple ) iter.next();
				String relation_name = Tuple.getPredicate().getBrowserText();
				
				for ( String thematic_role : System.getenv( "SUPPORTED_THEMATIC_ROLES" ).split( " " ) )	// Iterate over all thematic roles in the implementation.
				{			
					Pattern pattern = Pattern.compile( "is_" + thematic_role + "_for" );
					Matcher matcher = pattern.matcher( relation_name );
										
					if ( matcher.find() )					
					{
						relations.add( thematic_role );
						// System.out.println( "Detected semantic relation '" + relation_name + "' for object '" + object + "'." );
					}
				}
			}
		
			// System.out.println( "The entity has " + result.size() + " thematic relations." );
	
			if ( relations.contains( "AGENT" ) )
			{
				result = "ag";
				relations.remove( "AGENT" );
			}
			
			if ( relations.contains( "RECIPIENT" ) )
			{
				result = result + "_re";
				relations.remove( "RECIPIENT" );
			}
			
			if ( relations.contains( "THEME" ) )
			{
				result  = result + "_th";
				relations.remove( "THEME" );
			}
				
			if ( relations.contains( "THEME" ) )
			{
				result = result + "_th";
				relations.remove( "THEME" );
			}
		}
		
		return result;
	}

	
	/**
	*	Status: 
	*	
	*	This method returns all subjects that are related to some object
	*	via the input 'relation' in the asserted hierarchy.
	*/

	public Iterator <RDFResource> list_Asserted_Subjects( String relation )
	{		
		Iterator <RDFResource> result = new Vector <RDFResource> ().iterator();		
		
		try
		{
			result = owl_model.listSubjects( get_RDF_Property( relation ) );
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine the subjects for relation '" + relation + "' because: \n" + e + "\n" );
		}
		
		return result;
	}

	
	
	public Collection list_Objects( String subject, String relation )
	{
		/*
			Status: 
			
			This method returns a Collection of RDFResources (OWLIndividuals?) from 
			the asserted hierarchy that are related to the input RDFIndividual 
			'subject' via the relation 'relation'.
		*/
		
		Collection result = null;		
		
		try
		{
			result = owl_model.getTripleStoreModel().getPropertyValues( get_RDF_Resource( subject ), get_RDF_Property( relation ) );
		}
		catch( Exception e )
		{
			System.err.println( "The DIG_Query_Manager could not determine which objects the entity '" + subject + "' is related to via relation '" + relation + "' because: \n" + e + "\n" );
		}
		
		return result;
	}
	 
	
	
   	/**
   	 * 	This method returns an iterator over references consisting of subject-relation tuples 
	 *	in the OWL model that point to the input 'object'. The number of results to be handed 
	 *		
   	 */
	
   	public Iterator list_All_References_to( String object )
	{
		
		Iterator<Tuple> result = (new Vector<Tuple>()).iterator();		
		
		Object abc = get_RDF_Resource( object );
		if (abc != null)
			result = owl_model.listReferences( abc, -1 );
				
		return result;
	}
	    
	
   	
	/**
	* 	Status: Class 'Lexicalised.Concepts' has been created in the 
	*	upper ontology.
	*
	*	This method fills the hashmap 'lexicalisation_for' which maps 
	*	all lexicalised concepts to the vector of their lexicalisation strings.
	*/
   	
   	
	public void set_Concept_Lexicalisations()
	{
		System.out.println( "Individual found: " + get_Inferred_Individuals( "Lexicalised" ).size() );
		
		Iterator <RDFResource> iter = list_Asserted_Subjects( "has_Lexicalisation" );
	 	
		while ( iter.hasNext() )
		{
<<<<<<< DIG_Query_Manager.java
			System.out.println( "Hello!" );
	 		System.out.println( individual.toString() );
=======
			iter.next();
	 		System.out.println( "Individual found" );
>>>>>>> 1.14
//			String string = individual.getBrowserText();
//			System.out.println( "Found: " + string );
		}
		
//	 	while ( iter.hasNext() )
//		{		
//			RDFResource resource = (RDFResource ) iter.next();
//		 	System.out.println( lexicalisation_relation ); 	 	
//	 		System.out.println( resource.getBrowserText() ); 	 
//	 	}
	 	
// 	 	while ( iter.hasNext() )
// 		{
// 			System.out.println( "Got here OK." );
// 			OWLNamedClass subject = (OWLNamedClass) iter.next();			
// 			
// 			Iterator object_iterator = list_Objects( subject.getBrowserText(), lexicalisation_relation ).iterator();
// 			
// 			System.out.println( "Concept " + subject.getBrowserText() + " lexicalises as:\n" );
// 			
// 			while ( object_iterator.hasNext() )
// 			{
// 				RDFResource object = (RDFResource ) object_iterator.next();
// 				System.out.println( object.getBrowserText() ); 
// 			}
// 		}
	}
	    
	
	/**
	 * This method fills the HashMap 'Related' with all subjects in the 
	 *	OwL Model as key and all corresponding objects as their value.
	 */
	private void set_Context_Relations()
	{
		/*
			Status: 
			
			This method fills the HashMap 'Related' with all subjects in the 
			OwL Model as key and all corresponding objects as their value.
		*/
		
		context_relations = new HashMap();
		String subject, object;
		
		for ( Object subject_object : get_Asserted_Individual_Names() )		// Iterate subject candidates over all individuals in the asserted hierarchy.
		{
			subject = (String) subject_object;
							
			for ( Object object_object : get_Asserted_Individual_Names() )	// Iterate object candidates over all individuals in the asserted hierarchy.
			{
				object = (String) object_object;
				
				if ( is_Thematically_Related( subject, object ) )
				{
					context_relations.put( subject, object );
				}	
			}
		}
	}
	
	
	public HashMap get_Context_Relations()
	{
		return context_relations;
	}

	// End Method Definitions ============================================================
}