/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

%style nosort,before

%{
/* ------------------------------------------------------------------------- */
#include <config.h>
#ifdef SWIGTCL
#include <hooker.h>
#include <swig_utils.h>
#endif
#include <cdg.h>
#include <blah.h>
#include <netsearch.h>
#include <lexemgraph.h>
#include <constraintnet.h>
#include <parse.h>
#include <command.h>
#include <incrementalcompletion.h>
#include <eval.h>
#include <scache.h>
#include <write.h>
#include <levelvalue.h>
/* ------------------------------------------------------------------------- */
%}

%init %{
  cdgInitialize();
#ifdef SWIGTCL
  hooker_init(interp);
#endif
%}

%section "CDG language bindings"

%include pointer.i

%subsection "The BLAH language bindings"
%include blah.h

%include cdgtypemaps.i

#ifdef SWIGTCL
%subsection "HOOKS"
%include hooker.i
#endif

%subsection "CDG globals"
%readwrite
%include cdg.h
%readonly

%subsection "COMMANDS"
%include command.h

%subsection "INPUT"
%include input.i

%subsection "LEXEMGRAPH"
%include lexemgraph.h

%subsection "CONSTAINTNET"
%include constraintnet.h

%subsection "LEVELVALUE"
%include levelvalue.h

%subsection "NETSEARCH"
%include netsearch.h

%subsection "INCREMENTAL COMPLETITION"
%include incrementalcompletion.h

%subsection "STATISTICS"
%include statistics.h

%subsection "EVAL"
%include eval.h

%subsection "PARSE"
%include parse.h

%subsection "SCORE CACHE"
%include scache.h

%subsection "WRITE"
%include write.h

%subsection "MISC HELPERS"

%inline %{
/* 
 This function serves the sole purpose of finding a type`s direct successors 
 in a hierarchie`s distance-table. It is (at the moment) only used by the    
 the module allhierarchies.tcl                                               
*/
List listOfSons(Hierarchy hierarchy, int position_in_Hash_List_Of_Keys)
{
  Hashtable ht = hierarchy->types;
  List keys = hashListOfKeys(ht);
  List help, sons;
  Array a = hierarchy->distance;
  int x, y;
  Pointer item;

  item = listNthElement(keys, position_in_Hash_List_Of_Keys + 1);
  /* it is necessary to add 1 to position_in_Hash_List_Of_Keys, because */
  /* the function listNthElement starts to count with one */

  x = (int)hashGet(ht, item) - 1;
  /* 'x' is the type's x-coordinate in the distance table */

  sons = NULL;
  while (keys != NULL) {
    y = (int)hashGet(ht, keys->item) - 1;
    if ((int)arrayElement(a, x, y) == 1) {
      sons = listPrependElement(sons, keys->item);
    }
    help = keys;
    keys = keys->next;
    memFree(help);
  }
  return sons;
  listDelete(keys);
}
%}

%inline %{
/*
 cast a vector access to int
*/
int intVectorElementAt(Vector v, int i)
{
  return (int)vectorElement(v, i);
}
%}

