# usr/bin/perl -w

use strict;
use locale;
# Use of the current locale is needed to obtain correct sorting for German umlauts.

#	Based on the input from two or more files containing verb lemmata with their list of
#	semantic valences this script creates a unique and alphabetically sorted list of the
#	verb lemmata and their semantic valences. Note that if a lemma is contained in both files 
#	but contains different semantic valences in each file this script will fuse the input 
#	from both input files.	 
#
#	Invocation Parameters:
#	1. List of files from which to extract the lemmata and their white-space-separated 
#	semantic valences

my %semantic_valences;

# The following hash defines the colums in the output file for each semantic valence.

while( <> )
{
	if ( /^([\w]+)\s+(.+)/ )
	{
		my $lemma = $1;
		# print $lemma, "\n";
	
		my @valences = split /\s+/, $2;
		push @valences, @{ $semantic_valences{ $lemma } } if( defined $semantic_valences{ $lemma } );
		my %hash = &unique( @valences );
		@valences = keys %hash;
		# print "@valences\n\n";
		$semantic_valences{ $lemma } = [ @valences ];
	}
}

for my $key ( sort keys %semantic_valences ) 
{
	printf "%-20s %s\n\n", "$key", join "\t\t", ( sort @{ $semantic_valences{ $key } } );
}

##############################
#	Define Subroutines
##############################

sub unique
{
	my %hash;
	
	for my $element ( @_ )
	{
		$hash{ $element } = 1;
	}
	
	return %hash;
}

sub alphabetically
{
	# This subroutine sorts alphabetically according to the curent locale and is case-insensitive.
	"L$a" cmp "\L$b";
}
