/*
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/* -------------------------------------------------------------------------
 * @addtogroup Eval
 * @{
 */

#ifndef EVAL_H
#define EVAL_H

/* ---------------------------------------------------------------------- */
#include <blah.h>
#include "input.h"
#include "constraintnet.h"

/* ---------------------------------------------------------------------- */

#ifndef SWIG

/* -------------------------------------------------------------------------
 * Evaluation method
 *
 * This is the Type of the evaluation method which could be:
 * - interpreted
 * - compiled
 */
typedef enum {
  EMTInterpreted,
  EMTCompiled,
} EvalMethodType;
#endif

/* -------------------------------------------------------------------------
 * Generalized score for comparing any two structures.
 *
 * This structure counts both the total number of constraint violations
 * (called `conflicts' henceforth) and the number of hard conflicts (those
 * with score 0.0). The field soft is the product of the penalties of all
 * soft (non-zero) conflicts. Note that this structure allows different
 * criteria for comparing two analyses; the function bCompare() is provided
 * as one method, but others could be devised.
 */
typedef struct {
   int no;		     /**< number of conflicts */
   int hard;		     /**< number of hard conflicts */
   Number soft;		     /**< combined effect of soft conflicts */
} BadnessStruct;
/** A pointer to a BadnessStruct*/
typedef BadnessStruct *Badness;

/* -- VARIABLES --------------------------------------------------------- */

#ifndef SWIG
extern Constraint evalCurrentConstraint;
extern Formula evalCurrentFormula;

extern EvalMethodType evalEvaluationMethod;
extern EvalMethodType evalPeekValueMethod;

extern String static_string_id;
extern String static_string_grapheme;
extern String static_string_word;
extern String static_string_from;
extern String static_string_to;
extern String static_string_info;
extern String static_string_chunk_start;
extern String static_string_chunk_end;
extern String static_string_chunk_type;

extern Boolean evalInvalidArgEncountered;

#endif

extern int lock_counter;
extern int lock_width;
extern int *has_cache;

extern Boolean evalSloppySubsumesWarnings;

/* -- FUNCTIONS --------------------------------------------------------- */

#ifndef SWIG
extern void evalInitialize(void);
extern void evalFinalize(void);
extern Boolean evalValidateEvalMethod(String name, String value, int *var);

extern Value evalTerm(Term, Value, LexemGraph, Vector);

extern Boolean evalConstraint( Constraint c, ConstraintNet net, Vector context, ... );
extern Boolean evalUnaryConstraint(Constraint c, ConstraintNet net, Vector context,
				   LevelValue lv);
extern Boolean evalBinaryConstraint(Constraint c, ConstraintNet net, Vector context,
				    LevelValue lva, LevelValue lvb);
extern Number evalUnary(LevelValue lv, ConstraintNet net, Vector context,
                        Boolean use_cs_only,
                        Badness b, List *conflicts,
						Boolean writeScore);
extern Number evalBinary(LevelValue lva, LevelValue lvb,
                         ConstraintNet net, Vector context,
                         Boolean use_cs_only,
                         Badness b, List *conflicts);
#ifdef TERNARY
extern Number evalTernary(LevelValue lva, LevelValue lvb, LevelValue lvc,
                         ConstraintNet net, Vector context,
                         Boolean use_cs_only,
                         Badness b, List *conflicts);
extern Boolean evalTernaryConstraint(Constraint c, ConstraintNet net,
                             Vector context,
			     LevelValue lva, LevelValue lvb, LevelValue lvc);
extern Boolean evalTernaryList(int index, LevelValue lva, LevelValue lvb, LevelValue lvc,
			List *violated, List *conflicts, Number *score, Boolean use_cs_only, ConstraintNet net,
			Badness b, Vector context);
#endif
extern Number evalInContext(Vector LVs, Vector context, Badness b, List* conflicts);
extern Value peekValue( Value val, List path);
extern Badness bNew(int no, int hard, Number soft);
extern Badness bClone(Badness b);
extern void bCopy(Badness dest, Badness src);
extern Badness bAdd(Badness b, Number score);
extern Badness bSubtract(Badness b, Number score);
extern Badness bAddBadness(Badness a, Badness b);
extern Badness bSubtractBadness(Badness a, Badness b);
extern void bDelete(Badness b);
extern void bPrint(unsigned long mode, Badness b);
extern Number bScore(Badness b);
extern Boolean bEqual(Badness a, Badness b);
extern Boolean bCompare(Badness a, Badness b);
extern Badness bestBadness(void);
extern Badness worstBadness(void);
extern Boolean significantlyGreater(Number a, Number b);
extern void lock_tree(int width);
extern void unlock_tree(void);
#endif

/* ---------------------------------------------------------------------- */
#endif
/*@} */
/* don't insert anything after this #endif */

