/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 */

/*
 * File: predicates.h
 * Author: Ingo Schroeder
 * Purpose: header file for predicates.c
 * Birth: 17/3/97
 *
 */

#ifndef PREDICATES_H
#define PREDICATES_H

/* -- INCLUDES ---------------------------------------------------------- */
#include "cdg.h"
#include <blah.h>
#include "constraintnet.h"
#include "eval.h"

/* -- MACROS ------------------------------------------------------------ */
/* -- TYPE DEFINITIONS -------------------------------------------------- */
typedef struct {
  String functor;
  BooleanFunction *function;
  String functionName;
  Boolean is_context_sensitive;
  String arity;
  String doc;
} CDGPredicateStruct;
typedef CDGPredicateStruct* CDGPredicate;

/* -- FUNCTIONS --------------------------------------------------------- */
extern CDGPredicate predGetFunction(String functor);
extern String predGetFunctionName(String functor);
extern Boolean predStart(List args, LexemGraph lg, Vector context);
extern Boolean predStop(List args, LexemGraph lg, Vector context);
extern Boolean predFrontier(List a, LexemGraph lg, Vector context);
extern Boolean predRoot(List args, LexemGraph lg, Vector context);
extern Boolean predSpec(List args, LexemGraph lg, Vector context);
extern Boolean predVirtual(List args, LexemGraph lg, Vector context);
extern Boolean predNonSpec(List args, LexemGraph lg, Vector context);
extern Boolean predChunkHead(List args, LexemGraph lg, Vector context);
extern Boolean predPrint(List args, LexemGraph lg, Vector context);
extern Boolean predSubsumes(List args, LexemGraph lg, Vector context);
extern Boolean predExists(List args, LexemGraph lg, Vector context);
extern Boolean predCompatible(List args, LexemGraph lg, Vector context);
extern Boolean predHas(List args, LexemGraph lg, Vector context);
extern Boolean predIs(List args, LexemGraph lg, Vector context);
extern Boolean predOccur(List args, LexemGraph lg, Vector context);
extern Boolean predPartial(List l, LexemGraph lg, Vector context);
extern Boolean predCycle(List args, LexemGraph lg, Vector context);
extern Boolean predBetween(List args, LexemGraph lg, Vector context);
extern Boolean predUnder(List args, LexemGraph lg, Vector context);
extern Boolean predConnected(List args, LexemGraph lg, Vector context);
extern Boolean predSameVnSpec(List args, LexemGraph lg, Vector context);

//extern Boolean predUnderImpl(Pointer arg1, Pointer arg2, LexemGraph lg, Vector context);
extern Boolean predUnderImpl(Pointer arg1, Pointer arg2, LexemGraph lg, Vector context, int levelno);
extern Boolean predBetweenImpl(Pointer arg1, Pointer arg2, Pointer arg3, LexemGraph lg, Vector context);
Boolean predHasImpl(void **args);

/* ---------------------------------------------------------------------- */
#endif /* don't insert anything after this #endif */

