/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * Author: Ingo Schroeder <ingo@nats.informatik.uni-hamburg.de>
 * Purpose: Flex file for CDG parser
 * Date: 7/2/97
 */

/* ---------------------------------------------------------------------- */

changequote(`<<<',`>>>')

/* definitions */
%{
#include <config.h>
#include <strings.h>                           /* 990422, IS: index needed */
#include <errno.h>
#include <blah.h>
#include "cdg.h"
#include "hook.h"
#include "parsing.h"
#include "parser.h"
#include "command.h"                    /* 990318, IS: currentFile needed */
#include "input.h"            /* 990319, IS: inputRegisterFilename needed */

int parseLineNo = 1;
String parseCurrentFile;

static char stringChar;

#define FLEX_BUFFER_LENGTH 1024
#define YY_NO_INPUT

int my_yyinput(char *, int);
#define YY_INPUT(buf,result,max_size) (result = my_yyinput(buf,max_size))

%}

/* %option nounput */
/* VM_LATTICE is a lexical context for 
   VM Wort Hypothesis Graphs           */
%s CTXT_COMMENT CTXT_STRING VM_LATTICE

RE_TRUE   true
RE_FALSE  false
RE_LABEL  label
RE_LEVEL  level
RE_VIRTUALNODE virtualnode

/* Denotate the start and end of VM Lattice */
RE_BEGIN_LATTICE "BEGIN_LATTICE"
RE_END_LATTICE   "END_LATTICE"  

RE_MINUS  -
RE_IMPL   ->
RE_LPMI   <-
RE_BIIMPL <->
RE_GTEQ   >=
RE_LSEQ   <=
RE_NEQ    !=
RE_IDB    :=
RE_MAT    =~
RE_MAP    =>

RE_NUM    [0-9]+|[0-9]*"."[0-9]+

ifelse(OLDQUOTESTYLE,1,<<<
RE_STR    [a-zA-Z\"][a-zA-Z_0-9\"]*   
>>>, <<<
RE_STR    [a-zA-Z_\x80-\xff][a-zA-Z_0-9\x80-\xff]*
>>>)

/* Is a VM String - sourced from a set of samples */
VM_STR [a-zA-Z\"\$#][a-zA-Z0-9_:\.\-\"\'\$\?#]*

RE_TOP     "/\\"
RE_BOTTOM  "\\/"
RE_NONE    "||"
RE_INVERSE "~="
RE_IDENT   "=="

RE_SOURCEINFO "#line "[0-9]+(" \"".+"\"")?"\n"
RE_PRAGMA "#pragma ".*"\n"

%%
  int commentDepth = 0;
  char stringBuffer[FLEX_BUFFER_LENGTH];
  char *stringPtr = NULL;

<INITIAL>{RE_SOURCEINFO} { 
    /* get line number */
    sscanf( yytext, "#line %d", &parseLineNo );
    if ( '\"' == yytext[ strlen(yytext)-2 ] ) {     
      /* Ok, there is a new file filename */
      /* remove last quote and newline from string */
      yytext[ strlen(yytext)-2 ] = '\0';
      /* find first quote in yytext and skip over it */
      /* `index' is a built-in macro of m4, hide it */
      define(indexm4, defn(index)) undefine(index)
      parseCurrentFile = 
	inputRegisterFilename( parseResult, (String)( (int)index( yytext, '\"' ) + 1 ) );
      define(index, indexm4) undefine(indexm4)
    }
}

<INITIAL>{RE_PRAGMA} { 
  parseResult->pragmas =
    listAppendElement(parseResult->pragmas, strRegister(yytext+8));
  parseLineNo++;
}

<INITIAL>{RE_TRUE} { return CTRUE; }
<INITIAL>{RE_FALSE} { return CFALSE; }
<INITIAL>{RE_LABEL} { return LABEL; }
<INITIAL>{RE_LEVEL} { return LEVEL; }
<INITIAL>{RE_VIRTUALNODE} {return VIRTUALNODE;}

<INITIAL>{RE_BEGIN_LATTICE} {
/* Beginning of a VM Lattice */ 
        BEGIN(VM_LATTICE); 
        return TBEGIN_LATTICE; 
}

<VM_LATTICE>{RE_END_LATTICE} {
/* End of VM Lattice, 
   puts a semicolon on the end of the 
   VM Lattice */
        BEGIN(INITIAL);
        unput(';'); 
        return TEND_LATTICE; 
}

<VM_LATTICE>{VM_STR} {
/* A VM String has to be converted for the DAWAI parser
   i.e. >>"a<< is converted to >><<                    
*/
  yylval.STRING = strRegister(vm2String(yytext));
  return STR;
}

<INITIAL>{RE_IMPL} { return IMPL; }
<INITIAL>{RE_LPMI} { return LPMI; }
<INITIAL>{RE_BIIMPL} { return BIIMPL; }
<INITIAL>{RE_GTEQ} { return GTEQ; }
<INITIAL>{RE_LSEQ} { return LSEQ; }
<INITIAL>{RE_NEQ} { return NEQ; }
<INITIAL>{RE_IDB} { return ISDEFINEDBY; }
<INITIAL>{RE_MAT} { return MATCHES; }
<INITIAL>{RE_MAP} { return MAP; }

<INITIAL>{RE_TOP} { return CTOP; } 
<INITIAL>{RE_BOTTOM} { return CBOTTOM; }
<INITIAL>{RE_NONE} { return CNONE; }
<INITIAL>{RE_INVERSE} { return CINVERSE; }
<INITIAL>{RE_IDENT} { return CIDENT; }

<INITIAL,VM_LATTICE>{RE_MINUS} {
  yylval.NUMBER = atof(yytext);
  return MINUS;
}

<INITIAL,VM_LATTICE>{RE_NUM} {
  yylval.NUMBER = atof(yytext);
  return NUM;
}

<INITIAL>{RE_STR} {
  yylval.STRING = strRegister( yytext );
  return STR;
}

<INITIAL><<EOF>> { yyterminate(); }
ifelse(OLDQUOTESTYLE,1,<<<
<INITIAL>\' { 
>>>, <<<
<INITIAL>\'|\" { 
>>>)
  stringChar = *yytext;
  stringPtr = stringBuffer; 
  BEGIN(CTXT_STRING); 
}
ifelse(OLDQUOTESTYLE,1,<<<
<CTXT_STRING>\' {
>>>, <<<
<CTXT_STRING>\'|\" {
>>>)
   if ( stringChar == *yytext ) {
    BEGIN(INITIAL);
    *stringPtr = '\0';
    yylval.STRING = strRegister( stringBuffer );
    return STR;
  } else {
    if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) {
      *stringPtr++ = stringChar == '\'' ? '"' : '\'';    
    }
  }
}
<CTXT_STRING>\n {
  yywarning("string ended at end of line");
  parseLineNo++;
  BEGIN(INITIAL);
  *stringPtr = '\0';
  yylval.STRING = strRegister( stringBuffer );
  return STR;
}

<CTXT_STRING><<EOF>> {
  yywarning("string ended at end of file");
  BEGIN(INITIAL);
  *stringPtr = '\0';
  yylval.STRING = strRegister( stringBuffer );
  return STR;
}

<CTXT_STRING>\\n { 
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 )
    *stringPtr++ = '\n';
}
<CTXT_STRING>\\t { 
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    *stringPtr++ = '\t'; 
}
<CTXT_STRING>\\r {
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    *stringPtr++ = '\r';
}
<CTXT_STRING>\\b {
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 )
    *stringPtr++ = '\b';
}
<CTXT_STRING>\\f {
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    *stringPtr++ = '\f';
}
<CTXT_STRING>\\\n { parseLineNo++;  }
<CTXT_STRING>\\. { 
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    yytext++;
    *stringPtr++ = *yytext;
}
<CTXT_STRING>\\ { 
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    *stringPtr++ = '\\';
}

ifelse(OLDQUOTESTYLE,1,<<<
<CTXT_STRING>[^\\\n\']+ {
>>>, <<<
<CTXT_STRING>[^\\\n\'\"]+ {
>>>)
  char *yytextPtr = yytext;

  while ( *yytextPtr && strlen(stringBuffer) < FLEX_BUFFER_LENGTH - 1)
    *stringPtr++ = *yytextPtr++; 
}

       
<INITIAL,VM_LATTICE>"//"[^\n]*
<INITIAL,VM_LATTICE>"%"[^\n]*
<INITIAL,VM_LATTICE>[ \t]+
<INITIAL,CTXT_COMMENT,VM_LATTICE>\n { parseLineNo++; }
<INITIAL,CTXT_COMMENT>"/*" { commentDepth++; BEGIN(CTXT_COMMENT); }
<CTXT_COMMENT><<EOF>> {
  yywarning("comment ended at end of file");
  yyterminate();
}
<CTXT_COMMENT>[^*\n]*
<CTXT_COMMENT>"*"\n { parseLineNo++; }
<CTXT_COMMENT>"*"+[^*\n/]+
<CTXT_COMMENT>"*"+"/" { 
  commentDepth--;
  if ( commentDepth == 0 ) BEGIN(INITIAL);
}

. { return yytext[0]; }

%%
/* ---------------------------------------------------------------------- */
/* user code */

int yywrap() 
{
  return 1;
}

int my_yyinput(char *buf, int max_size)
{
  int result;

  if (YY_CURRENT_BUFFER_LVALUE->yy_is_interactive) {
    int c = '*', n;

    for (n = 0; n < max_size && (c = getc(yyin)) != EOF && c != '\n'; ++n)
      buf[n] = (char)c;
    if (c == '\n')
      buf[n++] = (char)c;
    if (c == EOF && ferror(yyin))
      YY_FATAL_ERROR("input in flex scanner failed");
    result = n;
  } else {
    do {
      errno = 0;
      clearerr(yyin);
      result = fread(buf, 1, max_size, yyin);
    } while (result == 0 && errno == EINTR);
    if (result == 0 && ferror(yyin))
      YY_FATAL_ERROR("input in flex scanner failed");
  }

  return result;
}
