/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * File: testing.c
 * Author: Ingo Schroeder
 *
 */

/* ---------------------------------------------------------------------- */
#include <config.h>

#include <sys/stat.h>
#include <sys/wait.h>
#include <stdio.h>
#include <limits.h>

#include "blah.h"
#include "input.h" /* for Input */
#include "parsing.h" /* for parseResult */
#include <string.h>
#include "hook.h" /* for cdgPrintf() */
#include "input.h" /* for data types */
#include "lexemgraph.h" /* for LexemGraph & LexemNode */
#include "levelvalue.h"
#include "eval.h"
#include "parse.h"
#include "tagger.h"
#include "chunker.h"
#include "write.h"
#include "command.h"
#include "eval.h"

/* ----------------------------------------------------------------------
   Undocumented command for testing
   1: determine base line for accuracy
*/
void tst1(int no, char **args)
{

  HashIterator hi;
  int all=0, base=0;
  char *level=no>=1 ? args[0] : NULL;
  char *label=no>=2 ? args[1] : "";
  char *use_pos=no>=3 ? args[2] : "yes";
  AnnoEntry ae;
  
  hi = hashIteratorNew(inputCurrentGrammar->annotations);
  while((ae = hashIteratorNextValue(hi))) {
    List m;
    for (m=ae->annos; m; m=m->next) {
      List n;
      Annotation a=m->item;
      
      
      for (n=a->specs; n; n=n->next) {
        Specification s=n->item;
        if (s->type!=STDep) { continue; }
        if (level && strcmp(level, s->kind)!=0) { continue; }
        all++;
        if (strcmp(label, s->name)!=0) { continue; }
        if (strcmp(use_pos, "yes")==0 && s->position>0) { continue; }
        base++;
      }
    }
  }
  hashIteratorDelete(hi);
  
  cdgPrintf(CDG_INFO, "TESTING: %d out of %d edges ", base, all);
  if (level) { cdgPrintf(CDG_INFO, "on level `%s' ", level); }
  cdgPrintf(CDG_INFO, "(%5.2f%%) in %d annotations are trivial.\n",
	    all>0 ? 100.0*(float)base/(float)all : 0.0,
	    hashSize(inputCurrentGrammar->annotations));
}

/* ----------------------------------------------------------------------
   Undocumented command for testing
   determine base line for accuracy
*/
void tst2(int no, char **args)
{
#if 0

  This used the "id" spec in the annotation to compare lexical entries, which
  is deprecated. Rewrite it to compare against Parses on a rainy day.
  
  List l;
  char *level=no>=1 ? args[0] : NULL;
  char *label=no>=2 ? args[1] : "";
  char *use_pos=no>=3 ? args[2] : "yes";
  double total_ratio=0.0;
  
  for (l=inputCurrentGrammar->annotations; l; l=l->next)
    {
      List m;
      AnnoEntry ae=l->item;
      Lattice lat = NULL; /* TODO this doesn't fix a possible coredump in lgNew
			   *      when there are no lattices
			   */
      LexemGraph lg;
      int i;
      double ratio=0.0;
      
      /* find word graph w for ae */
      for (m=inputCurrentGrammar->lattices; m; m=m->next)
	{
	  lat=m->item;
	  if (!strcmp(lat->id, ae->id)) { break; }
	  if (!strcmp(lat->annoId, ae->id)) { break; }
	}
      if (!m)
	{
	  cdgPrintf(CDG_INFO,
		    "Can't find wordgraph for annotation `%s', skipping\n",
		    ae->id);
	  continue;
	}
      lg=lgNew(lat, inputCurrentGrammar->lexicon);
      if (!lg)
	{
	  cdgPrintf(CDG_INFO,
		    "Can't create lexemgraph for wordgraph `%s', skipping\n",
		    lat->id);
	  continue;
	}

      /* run the simulation multiple times */
#define NO_ITER 100
      for (i=0; i<NO_ITER; i++)
	{
	  int j;
	  int all=0, base=0;
	  /* find random path p through lg */
	  for (j=lg->min; j<lg->max; /* NOT HERE */)
	    {
	      int count=0;
	      int choose;
	      int k;

	      for (k=0; k<vectorSize(lg->nodes); k++)
		{
		  LexemNode ln=vectorElement(lg->nodes, k);
		  if (ln->arc->from==j) { count++; }
		}
	      if (count<=0)
		{
		  cdgPrintf(CDG_INFO,
			    "Can't find lexem starting at %d in `%s', skipping\n",
			    j, lat->id);
		  j++;
		  continue;
		}
	      count--;
	      choose=(int)(drand48()*count+.5)+1;
	      for (k=0; k<vectorSize(lg->nodes); k++)
		{
		  LexemNode ln=vectorElement(lg->nodes, k);
		  if (ln->arc->from==j && --choose==0)
		    {
		      List n;
		      /* We've chosen ln as the next arc. */
		      j=ln->arc->to;

		      /* check for each arc a on path p whether a is in ae */
		      for (n=ae->annos; n; n=n->next)
			{
			  List o;
			  Annotation a=n->item;
			  char *id=NULL;
			  if (a->from!=ln->arc->from) { continue; }
			  if (a->to!=ln->arc->to) { continue; }
			  if (strcmp(a->word, ln->arc->word)) { continue; }
			  /* a is the annotation for ln */			  
			  for (o=a->specs; o; o=o->next)
			    {
			      Specification s=o->item;
			      if (s->type!=STTag) { continue; }
			      if (strcmp(s->kind, "id")!=0) { continue; }
			      id=s->name;
			    }
			  for (o=a->specs; o; o=o->next)
			    {
			      Specification s=o->item;
			      if (s->type!=STDep) { continue; }
			      if (level && strcmp(level, s->kind)!=0) { continue; }
			      all++;
			      if (strcmp(label, s->name)!=0) { continue; }
			      if (strcmp(use_pos, "yes")==0 && s->position>0) { continue; }
			      if (id && ln->lexem->description && strcmp(id, ln->lexem->description)!=0) { continue; }
			      base++;
			    }
			}		  
		      break;
		    }
		}
	    }
	  /*
	    cdgPrintf(CDG_INFO, "TESTING: all %4d base %4d `%s'\n",
		    all, base, lg->lattice->id);
	  */
	  ratio+=all>0 ? (double)base/(double)all : 0.0;
	  
	}
      cdgPrintf(CDG_INFO, "TESTING: ratio %10.4f average %8.5f%% `%s'\n",
		ratio, ratio/(double)NO_ITER*100.0, lg->lattice->id);
      lgDelete(lg);
      total_ratio+=ratio;
    }
  cdgPrintf(CDG_INFO, "\nTESTING: total ratio %10.4f average %8.5f%%",
		total_ratio, total_ratio*100.0/(NO_ITER*listSize(inputCurrentGrammar->annotations)));
  if (level)
    {
      cdgPrintf(CDG_INFO, " on level `%s'\n", level);
    } else {
      cdgPrintf(CDG_INFO, "\n", level);      
    }
#endif
}

/* ----------------------------------------------------------------------
   Undocumented command: detect constraints likely to fail on fragments
*/
void tst3(int no, char **args) {
  List l;
  Level Main = inputGetMainlevel(inputCurrentGrammar);
  String label;
  String id = NULL;
  int nrConstraints = hashSize(inputCurrentGrammar->constraints) + 1;
  Hashtable ht =
    hashNew(500, 0.7, hashStringHashFunction, hashStringEqualFunction);
  
  Boolean under(Vector v, LexemNode a, LexemNode b) {
    LevelValue lv;
    int index;
    if(a->arc->from == b->arc->from) {
      return TRUE;
    }
    index = inputCurrentGrammar->noOfLevels * a->arc->from + Main->no;
    lv = vectorElement(v, index);
    if(!lv->regents) {
      return FALSE;
    }
    return under(v, lv->regents->item, b);
  }
  
  /* gather our tools */
  if(!Main) {
    cdgPrintf(CDG_ERROR, "ERROR: no main level defined.\n");
    return;
  }
  if(no < 1) {
    cdgPrintf(CDG_ERROR, "ERROR: no fragment label specified.\n");
    return;
  }
  label = strRegister(args[0]);
  if(!listContains(Main->labels, label)) {
    cdgPrintf(CDG_ERROR, "ERROR: Level `%s' has no label `%s'.\n",
              Main->id, label);
    return;
  }
  if(-1 == taggerCategoryIndex) {
    cdgPrintf(CDG_ERROR, "ERROR: no `category' feature is defined.\n");
    return;
  }

  if(no > 1) {
    id = args[1];
  }
  
  cdgCtrlCTrapped = FALSE;
  cdgCtrlCAllowed = TRUE;
  
  /* replace each edge in the corpus with a NIL binding
     and see what happens. */
  for(l = inputCurrentGrammar->lattices; l != NULL; l = l->next) {
    Lattice lat = l->item;
    AnnoEntry ae = findAnnoForLattice(lat->id, TRUE);
    Parse p, pp;
    int i;
    LexemGraph lg;
    List vio = NULL;
    if(id && strcmp(id, lat->id)) continue;
    if(!ae) continue;
    if(!(p = parseFromAnno(ae))) continue;
    lg = lgNew(lat);
    parseDecorate(p, lg, ae);
    parseEval(p);

#ifdef DEBUGTST3
    cdgPrintf(CDG_DEBUG, "DEBUG: %s\n", lat->id);
#endif

    /* remember original violations */
    vio = listDeepClone(p->violations, (PointerFunction *)cvClone);

    /* We are now going to manipulate the parse, and that means inventing
       new LVs for it and deleting the old ones. Since shallow copies of
       these old LVs live in the elements of vio, we mustn't actually
       change p; so we clone p into pp and change that. */
    pp = parseClone(p, TRUE);
    for(i = 0; i < vectorSize(pp->LVs); i++) {
      LevelValue lv = vectorElement(pp->LVs, i);
      int index = parseIndex(pp, Main->id, ((LexemNode)lv->dependents->item)->arc->from);
      List m;
      String oldlabel;
      int oldregent;
      String cat = "";
      LexemNode oldDependent;
      Value v;
      if(lv->level != Main) continue;
      if(!spec(lv->regents)) continue;
      oldlabel = lv->label;
      oldregent = ((LexemNode)lv->regents->item)->arc->from;
      oldDependent = lv->dependents->item;
      v = ((LexemNode)lv->dependents->item)->lexem->values[taggerCategoryIndex];
      if(v->type == VTString) {
        cat = strCopy(v->data.string);
      }
#ifdef DEBUGTST3
      cdgPrintf(CDG_DEBUG, "DEBUG: @%d %s\n", i, ((LexemNode)lv->dependents->item)->arc->word);
#endif
      parseSwapLabel(pp, index, label, TRUE);
      parseShiftEdge(pp, index, -1, TRUE);
      for(m = pp->violations; m != NULL; m = m->next) {
        ConstraintViolation cv = m->item;
        if(!cvContains(vio, cv)) {
          Vector v;

          /* only report conflicts that occur
             within the scope of this word */
          if(!under(pp->LVs, cv->lv1->dependents->item, oldDependent)) {
            continue;
          }
          if(cv->lv2 && !under(pp->LVs, cv->lv2->dependents->item, oldDependent)) {
            continue;
          }            

          if(NULL != (v = hashGet(ht, cat))) {
            int i = (int)vectorElement(v, cv->constraint->no);
            i++;
            vectorSetElement(v, (Pointer)i, cv->constraint->no);
          } else {
            v = vectorNew(nrConstraints);
            vectorSetAllElements(v, (Pointer)0);
            vectorSetElement(v, (Pointer)1, cv->constraint->no);
            hashSet(ht, cat, v);
          }
          
          
#ifdef DEBUGTST3
            cdgPrintf(CDG_DEBUG,
                      "DEBUG:  `%s' introduced `%s'\n",
                      cat, cv->constraint->id);
#endif 
        }
      }
      parseShiftEdge(pp, index, oldregent, TRUE);
      parseSwapLabel(pp, index, oldlabel, TRUE);
    }
    lgDelete(lg);
    parseDelete(pp);
    listForEachDelete(vio, cvDelete);
    parseDelete(p);

    if(cdgCtrlCTrapped) {
      break;
    }
  }

  {
    HashIterator hi = hashIteratorNew(ht);
    Vector v;
    String key;
    while (NULL != (key = hashIteratorNextKey(hi))) {
      HashIterator hi2 = hashIteratorNew(inputCurrentGrammar->constraints);
      Constraint c;
      v = hashGet(ht, key);
      while (NULL != (c = hashIteratorNextValue(hi2))) {
        int i = (int)vectorElement(v, c->no);
        if(i) {
          cdgPrintf(CDG_DEBUG, "%s:%s:%d\n", key, c->id, i);
        }
      }
      hashIteratorDelete(hi2);
    }
    hashIteratorDelete(hi);
  }
  hashForEachFreeValue(ht, vectorDelete);
}



/* undocumented: show where constraints apply */

/* forward declaration -- evalFormula not normally exported */
Boolean evalFormula(Formula f, LexemGraph lg, Vector context);

void tst5(int no, char **args) {

  HashIterator hi = hashIteratorNew(inputCurrentGrammar->annotations);
  List l;
  int i;
  AnnoEntry ae;
  List constraints = hashListOfKeys(inputCurrentGrammar->constraints);
  int nrConstraints = listSize(constraints);
  int apply[nrConstraints], fail[nrConstraints];
  int counter = 0;

  for(i = 0; i < nrConstraints; i++) {
    apply[i] = fail[i] = 0;
  }

  while (NULL != (ae = hashIteratorNextValue(hi))) {
    Parse parse = parseFromAnno(ae);
    Lattice lat;
    LexemGraph lg;

    cdgPrintf(CDG_DEBUG, "[%d]\n", ++counter);

    if (!parse) continue;
    lat = findLattice(ae->lattice);
    if (!lat) continue;
    lg = lgNew(lat);
    if(!lg) continue;
    if (!parseDecorate(parse, lg, ae)) continue;

    /* check each constraint */
    for(l = constraints; l != NULL; l = l->next) {
      String id = l->item;
      Constraint c = hashGet(inputCurrentGrammar->constraints, id);
      
      /* ignore inactive constraints */
      if (!c->section->active || !c->active) continue;

      /* ignore variable-strength constraints */
      if(c->penaltyTerm) {
        continue;
      }

      /* try unary constraints on each edge */
      if(listSize(c->vars) == 1) {
        int i;
        for(i = 0; i < vectorSize(parse->LVs); i++) {
          LevelValue lv = vectorElement(parse->LVs, i);
          if(!lv) continue;

          if(matchUnarySignature(lv, c->sig)) {
            Formula premise;
            Formula conclusion;

            /* instantiate */
            VarInfo vi = c->vars->item;
            vi->levelvalue = lv;
            evalCurrentConstraint = c;
            
            /* test */
            if(FTImplication == c->formula->type) {
              premise = c->formula->data.junction.arg1;
              conclusion = c->formula->data.junction.arg2;
            } else {
              premise = NULL;
              conclusion = c->formula;
            }
            if(!premise || evalFormula(premise, parse->lg, parse->LVs)) {
              apply[c->no]++;
              if(!evalFormula(conclusion, parse->lg, parse->LVs)) {
                fail[c->no]++;
              }
            }
          }
        }
      }
   
      /* try binary constraints on each edge pair */
      else {
        int i;
        for(i = 0; i < vectorSize(parse->LVs); i++) {
          int j;
          LevelValue lvi = vectorElement(parse->LVs, i);
          if(!lvi) continue;
          for(j = 0; j < vectorSize(parse->LVs); j++) {
            LevelValue lvj = vectorElement(parse->LVs, j);
            int k;
            if(!lvj) continue;
            if(lvi == lvj) continue;
            for(k = 0; k < 2; k++) {
              LevelValue lva, lvb;
              if(k == 0) {
                lva = lvi;
                lvb = lvj;
              } else {
                lva = lvj;
                lvb = lvi;
              }
              
              if(matchBinarySignature(lva, lvb, c->sig)) {
                Formula premise;
                Formula conclusion;
            
                /* instantiate */
                VarInfo vi = c->vars->item;
                vi->levelvalue = lva;
                vi = c->vars->next->item;
                vi->levelvalue = lvb;
                evalCurrentConstraint = c;

                /* test */
                if(FTImplication == c->formula->type) {
                  premise = c->formula->data.junction.arg1;
                  conclusion = c->formula->data.junction.arg2;
                } else {
                  premise = NULL;
                  conclusion = c->formula;
                }

                if(!premise || evalFormula(premise, parse->lg, parse->LVs)) {
                  apply[c->no]++;
                  if(!evalFormula(conclusion, parse->lg, parse->LVs)) {
                    fail[c->no]++;
                  }
                }
              }
            }
          }
        }
      }
    }
    parseDelete(parse);
  }


  for(l = constraints; l != NULL; l = l->next) {
    String id = l->item;
    Constraint c = hashGet(inputCurrentGrammar->constraints, id);

    if (!c->section->active || !c->active) continue;
    if(c->penaltyTerm) continue;

    cdgPrintf(CDG_INFO,
              "%s: applied %d, held %d, failed %d\n",
              c->id, apply[c->no], apply[c->no] - fail[c->no], fail[c->no]);
  }

  listDelete(constraints);
}


/* show what constraints are violated
   by breaking the sentence at specific points. */
void tst7(int no, char **args)
{
  AnnoEntry ae = NULL;
  Parse p;
  List l;
  int i = 0;
  List boundaries = NULL;
  LexemGraph lg;
  Lattice lat = NULL;

  if (no < 1) {
    cdgPrintf(CDG_INFO, " usage: testing 7 <annoid>\n");
    return;
  }

  lat = findLattice(strRegister(args[0]));
  ae = findAnnoForLattice(lat->id, TRUE);
  if (!ae || !lat) {
    cdgPrintf(CDG_ERROR, "ERROR: no annotation `%d'.\n", args[0]);
    return;
  }

  lg = lgNew(lat);
  p = parseFromAnno(ae);
  if (!p || !parseDecorate(p, lg, ae)) {
    cdgPrintf(CDG_INFO, "ERROR: could not decorate this parse.\n");
    lgDelete(lg);
    return;
  }
  parseEval(p);
  
  i = 1;
  while (i < no) {
    int j;

    sscanf(args[i], "%d", &j);
    boundaries = listAppendElement(boundaries, (Pointer) j);
    i++;
  }

  /* direct all edges to NIL if they cross boundaries */
  for (i = 0; i < vectorSize(p->LVs); i++) {
    LevelValue lv = vectorElement(p->LVs, i);
    int index = lvIndex(lv);
    String label;

    if (!spec(lv->regents)) {
      continue;
    }
    if (lv->level == inputGetMainlevel(inputCurrentGrammar)) {
      label = strRegister("ROOT");
    } else {
      label = strRegister("");
    }
    for (l = boundaries; l; l = l->next) {
      int j = (int)l->item;

      if ((((LexemNode)lv->regents->item)->arc->from > j && ((LexemNode)lv->dependents->item)->arc->from <= j)
	  || (((LexemNode)lv->dependents->item)->arc->from > j && ((LexemNode)lv->regents->item)->arc->from <= j)) {
	parseShiftEdge(p, index, -1, TRUE);
	parseSwapLabel(p, index, label, TRUE);
      }
      break;
    }
  }
  for (l = p->violations; l != NULL; l = l->next) {
    ConstraintViolation cv = l->item;
    if (cv->penalty < 0.2) {
      cvPrint(CDG_INFO, cv);
      cdgPrintf(CDG_INFO, "\n");
    }
  }

}
/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
