/*
 * Copyright (C) 1997-2009 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Niels Beuck
 * Birth: 23.10.09
 *
 */

#ifndef VIRTUALNODE_H_
#define VIRTUALNODE_H_

#include <blah.h>
#include "input.h"
#include "cdg.h"
#include "constraintnet.h"
#include "transform.h"

/** Types ***********************************************************************/

/**
 * This is an extension to an lexemgraph containing all virtual nodes,
 * i.e. those nodes that are not based on the input.
 * Virtual Nodes are used for anticipation in incremental parsing
 *
 * NIL and NONSPEC are not part of this, as they are just attachment points,
 * and don't have a domain themselves
 *
 */
struct VirtualLGExtensionStruct {
  LexemGraph lg; 						/** the lg this is an extensions to */
  Vector virtualNodes;                  /** vector of virtual (grapheme) nodes */
  //Vector lexemNodes;                  /** vector of lexeme nodes */
  int vStartID;							/** first id in the lg that is virtual */
  //int maxNonVirtual; 					/** like lg->max but not counting arcs of virtual nodes */
  //int nextId;                         /** next id to be distributed */
} ;
// typedef VirtualLGExtensionStruct *VirtualLGExtension; // defined in cdg.h to prevent circular definitions

struct VirtualNodeSpecStruct {
	String name;				/** name that will appear as lexical form */
	Boolean multiple;			/** if there can be multiple copies of this node or not */
	List values;				/** list of postags that this node can represent */
	Number penalty;				/** penalty if this node is used */
	int copies;					/** how many copies should be generated in advance **/
} ;
//typedef VirtualNodeSpecStruct *VirtualNodeSpec; // defined in cdg.h to prevent circular definitions

typedef struct {
	String id;
	VirtualNodeSpec spec;
	List graphemNodes;
} VirtualNodeStruct;
typedef VirtualNodeStruct *VirtualNode;


/** Variables ***********************************************************************/
extern Boolean useVirtualNodes;
extern Boolean matchVirtualNodes;
extern Boolean orderVirtualNodes;
extern float   virtMatchingFactor;

/** Procedures **********************************************************************/
extern void virtualInitialize();
extern VirtualLGExtension veNew(LexemGraph lg);
extern VirtualNode vnNew(VirtualNodeSpec vspec);
extern void lgAddGraphemNode(LexemGraph lg, GraphemNode gn);
extern GraphemNode lgAddVirtualNode(LexemGraph lg, VirtualNodeSpec spec, int n);
extern int cnGetFirstVirtualIndex(ConstraintNet net);
extern void cnManageVirtualNodes(ConstraintNet net, Analysis a, ByteVector exceptions, Vector freezeCount);
extern void cnInitVirtualNodes(ConstraintNet cn, List specs, ByteVector exceptions, Vector freezeCount);
extern Boolean gnIsVirtual(GraphemNode gn);
extern VirtualNodeSpec vnsNew(String name,Number penalty, List values, int copies);
extern VirtualNodeSpec vnsFromAnno(Annotation an);
extern VirtualLGExtension veClone(VirtualLGExtension ve);
extern void veDelete(VirtualLGExtension ve);
extern Analysis aBestMatchingReplacement(Analysis a, GraphemNode newNode, Number threshold);
extern Boolean gnIsUnusedVirtualNode(GraphemNode gn, Analysis a);
extern VirtualNode veGetVirtualNode(VirtualLGExtension ve, String id);

#endif /* VIRTUALNODE_H_ */
