/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * The functions tested in the List module
 *
 * listAppendElement
 * listClone
 * listContains
 * listIsEqual
 * listAddUniqueElement
 * listDeleteLastElement
 * listAppendList
 * listLastElement
 * listSort
 * listInsertSorted
 * listReverse
 * listNthElement
 * listDeleteElement
 * vecorToList
 * listToVector
 * listIndex
 */


#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <blah.h>
#include <assert.h>

char *a[] = {
  "null", "eins", "zwei", "drei", "vier", "fuenf", "sechs", "sieben",
  "acht", "neun", "zehn", "elf", "zwoelf", "dreizehn", "vierzehn"
};

char *b[] = {
  "fuenfzehn","sechzehn","seibzehn","achtzehn","neunzehn"
 };

Boolean cmpString(String a, String b) {
  if (a == NULL)
    return (FALSE);
  return (b == NULL || strcmp(a, b) < 0);
}

int main(int argc, char **argv)
{
  List list = NULL, list2 = NULL, reverse;
  List listFromVector, listAfterClone;
  int i = 1, j, l2;
  Vector v = vectorNew(15);

  blahInitialize();

  /* 
   * ----------------------------------------
   * Test for listAppendElement
   * Creates lists list,list2
   */
  for (i = 0; i < 15; i++) {
    list = listAppendElement(list, a[i]);
  }
  assert(listSize(list) == 15);
  for (i = 0; i < 15; i++)
    assert(listContains(list, a[i]));

  for (j = 0; j < 5; j++) {
    list2 = listAppendElement(list2, b[j]);
  }
  assert(listSize(list2) == 5);
  for (j = 0; j < 5; j++)
    assert(listContains(list2, b[j]));

  /* 
   * ----------------------------------------
   * Test for listClone,listContains
   * ListAfterClone is clone of list2
   */

  listAfterClone = listClone(list2);
  assert(listSize(listAfterClone) == 5);
  for (i = 0; i < 5; i++)
    assert(listContains(listAfterClone, b[i]));

  /* 
   * ----------------------------------------
   * Test for listIsEqual
   * Compares the list2 and listAfterClone
   */
  
  assert(listIsEqual(list2, list2));
  assert(listIsEqual(list, list2) == FALSE);
  assert(listIsEqual(list2, listAfterClone));

  /* 
   * ---------------------------------------
   * Test for listAddUniqueElement
   * Adds an item into listAfterClone
   */

  assert(listContains(listAfterClone, a[2]) == FALSE);
  listAddUniqueElement(listAfterClone, a[2]);
  assert(listContains(listAfterClone, a[2]) == TRUE);
  assert(listAddUniqueElement(listAfterClone, a[2]) == listAfterClone);

  /* 
   * ----------------------------------------
   * Test for listDeleteLastElement
   */

  l2 = listSize(listAfterClone);
  assert(strcmp(listLastElement(listAfterClone), "zwei") == 0);
  listDeleteLastElement(listAfterClone);
  assert(listSize(listAfterClone) == (l2 - 1));
  assert(strcmp(listLastElement(listAfterClone), "neunzehn") == 0);
  listDelete(listAfterClone);

  /* 
   * -----------------------------------------
   * Test listAppendList,listLastElement
   * appending list to list 2.
   */

  listAppendList(list2, list);
  assert(listSize(list2) == 20);
  assert(strcmp(listLastElement(list2), "vierzehn") == 0);

  /* 
   * ----------------------------------------
   * Test listSort
   */
  {
    List sortedList = listSort(list, cmpString);

    list = sortedList;
    assert(listSize(list) == 15);
    assert(strcmp(listLastElement(list), "zwoelf") == 0);

  }

  listDelete(list2);
  /* 
   * -----------------------------------------
   * Test listReverse
   */

  reverse = listReverse(list);
  assert(listSize(reverse) == 15);
  assert(strcmp(listLastElement(reverse), "acht") == 0);

  /* 
   * ----------------------------------------
   * ListInsertSorted,listNthElement
   */

  listInsertSorted(list, "fuenfzehn", cmpString);
  assert(listSize(list) == 16);
  assert(strcmp(listNthElement(list, 6), "fuenf") == 0);
  assert(strcmp(listNthElement(list, 7), "fuenfzehn") == 0);
  assert(strcmp(listNthElement(list, 8), "neun") == 0);

  /* 
   * -----------------------------------------
   * listDeleteElement
   */

  listDeleteElement(list, "fuenfzehn");
  assert(listSize(list) == 15);
  assert(strcmp(listNthElement(list, 6), "fuenf") == 0);
  assert(strcmp(listNthElement(list, 7), "neun") == 0);

  /* 
   * -----------------------------------------
   * listToVector
   */

  v = listToVector(list);
  assert(vectorSize(v) == 15);
  for (i = 0; i < vectorSize(v); i++)
    assert(vectorContains(v, a[i]));

  /* 
   * ----------------------------------------
   * vectorToList,listIndex
   */

  listFromVector = vectorToList(v);
  assert(listSize(listFromVector) == 15);
  for (i = 0; i < listSize(listFromVector); i++) {
    assert(listContains(listFromVector, a[i]));
    assert(listIndex(listFromVector, a[i]) == listIndex(list, a[i]));
  }

#if 0
  /* 
   * ----------------------------------------
   * PRINT a list
   */
  for (l = list; l; l = l->next) {
    printf("%s ", l->item);
  }

  printf("\n");
#endif

  blahFinalize();
  return 0;
}

