/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* The functions of ringbuffer tested are
 *
 * rbNew
 * rbDelete
 * rbIsEmpty
 * rbIsFull
 * rbAddTopElement
 * rbAddBottomElement
 * rbTopPeek
 * rbBottomPeek
 * rbSize
 * rbCapacity
 * rbRemoveTopElement
 * rbRemoveBottomElement
 * rbCopy
 * rbClone
 * rbClear
 *
 * Functions not tested are
 *
 * rbForEachWithData
 * rbcontains
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <assert.h>
#include <blah.h>

#define COUNT 10

int main(int argc, char **argv)
{
  RingBuffer rb,rb2,rbAfterClone;
  int i, n;

  blahInitialize();

  /* ----------------------------------------
   * test rbNew
   */
  rb = rbNew(COUNT);
  rb2= rbNew(COUNT);
  rbAfterClone = rbNew(COUNT);

  assert(rb != NULL);

  /*
   * test capacity
   */
  assert(COUNT == rbCapacity(rb));

  /* ----------------------------------------
   * test IsEmpty
   */
  assert(rbIsEmpty(rb));

  /* ----------------------------------------
   * test addTopElement, TopPeek, BottomPeek
   */

  for (i = 0; i < COUNT; i++) {
    assert(i + 1 == (int)rbAddTopElement(rb, (Pointer) i));
    assert(i == (int)rbTopPeek(rb));
    assert(0 == (int)rbBottomPeek(rb));
  }

  /* ----------------------------------------
   * test isFull
   */

  assert(rbIsFull(rb));
  assert(rbSize(rb) == COUNT);


  /* ----------------------------------------
   * test clear
   */

  rbClear(rb);
  assert(rbIsEmpty(rb));
  assert(rbSize(rb) == 0);


  /* ----------------------------------------
   * Test addBottomElement, TopPeek, BottomPeek
   */

  for (i = 0; i < COUNT; i++) {
    assert(i + 1 == (int)rbAddBottomElement(rb, (Pointer) i));
    assert(0 == (int)rbTopPeek(rb));
    assert(i == (int)rbBottomPeek(rb));
  }


  /* test isFull */
  assert(rbIsFull(rb));
  assert(rbSize(rb) == COUNT);


  /* ----------------------------------------
   * test removeTop
   */

  for (i = 0; i < COUNT; i++) {
    assert(i == (int)rbRemoveTopElement(rb));
  }


  /* ----------------------------------------
   * Test removeBottom
   */

  for (i = 0; i < COUNT; i++) {
    assert(i + 1 == (int)rbAddTopElement(rb, (Pointer) i));
  }
  assert(rbIsFull(rb));
  assert((int)rbTopPeek(rb) == COUNT - 1);
  assert((int)rbBottomPeek(rb) == 0);
  for (i = 0; i < COUNT; i++) {
    assert(i == (int)rbRemoveBottomElement(rb));
  }
  assert(rbIsEmpty(rb));


  /* ----------------------------------------
   * mixed test 
   */

  for (i = 0; i < COUNT; i++) {
    assert(i + 1 == (int)rbAddTopElement(rb, (Pointer) i));
  }
  for (i = 0; i < COUNT; i++) {
    n = (int)rbRemoveBottomElement(rb);
    assert(rbSize(rb) == COUNT - 1);
    rbAddTopElement(rb, (Pointer) n);
    assert(rbSize(rb) == COUNT);
  }
  assert(rbIsFull(rb));

  /* ----------------------------------------
   * Test rbCopy
   */
  rb2=rbCopy(rb2,rb);
  assert((int)rbTopPeek(rb)==(int)rbTopPeek(rb2));
  assert((int)rbBottomPeek(rb)==(int)rbBottomPeek(rb));

  /* ----------------------------------------
   * Test rbClone
   */
  rbAfterClone=rbClone(rb);
  assert((int)rbTopPeek(rb)==(int)rbTopPeek(rbAfterClone));
  assert((int)rbBottomPeek(rb)==(int)rbBottomPeek(rbAfterClone));
  
  blahFinalize();
  return 0;
}
