#!/usr/bin/env perl
# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# cdg/utils/txt2lat.pl
#
# Produces CDG lattices from flat files with on eword per line.
use warnings;
use strict;

use Getopt::Std;
getopts("adf:hlp:s");
our $opt_a;
our $opt_d;
our $opt_h;
our $opt_f;
our $opt_l;
our $opt_p;
our $opt_s;

die "
Usage: $0 [input file]

       -a  also produce an empty Annotation
       -d  Distribute lattices into numbered subdirectories
       -f  Force lattice name (only useful for a single sentence per run)
       -h  show this Help
       -l  use Line-based format
       -p  specify Prefix for lattice names
       -s  Suppress lattice names in output

" if $opt_h;

our $prefix = $opt_p ? "$opt_p-" : '';

our $keywords = 'true|false|level|label';

our $counter = 0;

sub open_lattice_file {
  my $x = $counter / 10000;
  my $dir = sprintf "%03d", $x;
  if( ! -d $dir) {
    mkdir $dir or die "Can't mkdir $dir: $!\n";
  }
  my $filename = "$dir/$dir.lat";
  open(OUT, ">$filename") or die "Can't write `>$filename': $!\n";
}

sub close_lattice_file {
  close(OUT);
}

# Produce lattice output for the passed array and empty it.
sub write_lattice ($) {
  my $ref = shift;
  my @x = @$ref;
  return unless @x;

  if($x[0] =~ /^\@name=(.+)/) {
    $prefix="$1-";
  }

  # insert quotes only where necessary
  my @quoted;
  for (@x) {

    next if /^\@/;

    if (/[^a-zA-Z]|^$keywords$/) {

      # escape internal quotes
      s/([\'\"\\])/\\$1/g;

      # add external quotes
      $_ = "'$_'";
    }
    push @quoted, $_;
  }
  if(!@quoted) {
    return;
  }

  $counter++;
  if($opt_d && 0 == $counter % 10000) {
    close_lattice_file();
    open_lattice_file();
  }

  my $name = "${prefix}s$counter";
  if(!$opt_s) {
    if($opt_f) {
      print OUT "'$opt_f' : "
    } else {
      print OUT "'$name' : "
    }
  }
  print OUT join(', ', @quoted);
  print OUT ";\n\n";
  if ($opt_a) {
    print OUT "'null-$name' : '$name' <->\n";
    my @lines=();
    for (my $i=0; $i<$#quoted; $i++) {
        push @lines,"$i ".($i+1)."\t\t$quoted[$i] SYN -> 'S' -> 0 REF -> '' -> 0";
      }
    print OUT join(",\n",@lines);
    print OUT ";\n\n";
  }

  @$ref = ();
}

# the sentence under construction
our @sentence = ();

if($opt_d) {
  open_lattice_file();
} else {
  *OUT = *STDOUT;
}

if($opt_l) {
  while(<>) {
    chomp;
    next if /^$/;
    @sentence = split(/ /);
    write_lattice(\@sentence);
  }
} else {
  while (<>) {
    chomp;
    if(/^$/) {
      write_lattice(\@sentence);
    } else {
      push @sentence, $_;
    }
  }
  write_lattice(\@sentence);
}

if($opt_d) {
  close_lattice_file();

  $prefix =~ s/-$//;
  open(LAT, ">$prefix.lat") or die "Can't write `>$prefix.lat': $!\n";
  my $cwd = `pwd`;
  chomp $cwd;
  for(0..int($counter / 10000)) {
    printf LAT "#pragma load $cwd/%03d/%03d.lat\n", $_, $_;
  }
  close(LAT);

  system "echo $counter > MAX";
}



