# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## FileSelector - dialog to select files and directories.
## This class implements an improved file selection dialog using the classes
## VisDirectory and VisFile.
##
## \author Michael Daum 
##
## ----------------------------------------------------------------------------
class FileSelector {
  inherit iwidgets::Shell

  # -- variables -------------------------------------------------------------
  public variable showHidden 0
  public variable filter "*"
  public variable directory ""
  public variable mode "multi"
  public variable selection ""

  private variable _history {}
  private variable _historyIndex 0
  private variable _useHistory 1
  private variable _currentFilter "*"

  # -- methods ---------------------------------------------------------------
  public method show {{dirName ""}}; ## \type TclString
  public method activate {args}; ## \type TclList

  constructor {args} {}; ## \type TclList
  
  private method _okCommand {}
  private method _cancelCommand {}
  private method _directoryCommand {}
  private method _homeCommand {}
  private method _upCommand {}
  private method _reloadCommand {}
  private method _selectionCommand {}
  private method _chdirHandle {}
  private method _backwardHistory {}
  private method _forwardHistory {}
  private method _updateHistory {}
  private method _selectHandle {}
  private method _newCommand {}
  private method _deleteCommand {}
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body FileSelector::constructor {args} {

  # the filearea
  itk_component add visDirectory {
    VisDirectory $itk_interior.visDirectory \
	-hscrollmode dynamic \
	-vscrollmode none \
	-sbwidth 12 \
	-autoresize 0 \
	-chdircommand [code $this _chdirHandle] \
	-selectioncommand [code $this _selectHandle] \
	-finishcommand [code $this _okCommand]
  } {}

  # the toolbar
  itk_component add toolbar {
    iwidgets::toolbar $itk_interior.toolbar \
      -balloonfont {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} \
      -balloonbackground lightyellow \
      -borderwidth 0 \
  } {}

  $itk_component(toolbar) add FlatButton backward \
	-image [YadaImages::get backward] \
	-balloonstr "back" \
	-command [code $this _backwardHistory] \
	-state disabled
  
  $itk_component(toolbar) add FlatButton forward \
	-image [YadaImages::get forward] \
	-balloonstr "forward" \
	-command [code $this _forwardHistory] \
	-state disabled

  
  $itk_component(toolbar) add FlatButton upward \
	-image [YadaImages::get up] \
	-balloonstr "upwards" \
	-command [code $this _upCommand]
  
  $itk_component(toolbar) add FlatButton home \
	-image [YadaImages::get home] \
	-command [code $this _homeCommand] \
	-balloonstr "go home"

  $itk_component(toolbar) add FlatButton reload \
	-image [YadaImages::get reload] \
	-balloonstr "reload" \
	-command [code $this _reloadCommand]
  
  $itk_component(toolbar) add FlatButton new \
	-image [YadaImages::get lamp] \
	-balloonstr "new" \
	-command [code $this _newCommand]

  $itk_component(toolbar) add FlatButton delete \
	-image [YadaImages::get delete] \
	-balloonstr "delete" \
	-command [code $this _deleteCommand]

  itk_component add directory {
    iwidgets::combobox $itk_interior.directory \
      -textvariable [scope directory] \
      -editable 0 \
      -selectioncommand [code $this _directoryCommand] \
  } {}

  itk_component add ff {
    frame $itk_interior.ff
  }

  itk_component add selection {
    iwidgets::combobox $itk_component(ff).selection \
      -labeltext "Selection:" \
      -textvariable [scope selection] \
      -selectioncommand [code $this _selectionCommand] \
      -completion false
  } {}


  itk_component add filter {
    iwidgets::combobox $itk_component(ff).filter \
      -labeltext "Filter:" \
      -textvariable [scope _currentFilter] \
      -selectioncommand [code $this show] \
      -completion false
  } { }

  itk_component add showHidden {
    checkbutton $itk_component(ff).showHidden \
      -text "Show hidden" \
      -highlightthickness 0 \
      -variable [scope showHidden] \
      -command [code $this show]
  }


  itk_component add okButton {
    button $itk_component(ff).okButton \
      -text "Ok" \
      -command [code $this _okCommand]
  }

  itk_component add cancelButton {
    button $itk_component(ff).cancelButton \
      -text "Cancel" \
      -command [code $this _cancelCommand]
  }

  itk_component add message {
    iwidgets::messagedialog $itk_interior.message \
      -title "Yada - Caution"  \
      -modality application \
      -master $itk_component(hull)
  } { }
  $itk_component(message) buttonconfigure OK -text "Yes"
  $itk_component(message) buttonconfigure Cancel -text "No"
  $itk_component(message) hide Apply
  $itk_component(message) hide Help

  # packing
  grid rowconfigure $itk_interior 1 -weight 1
  grid columnconfigure $itk_interior 1 -weight 1
  grid $itk_component(toolbar) -row 0 -column 0 -sticky ew -padx 0 -pady 0
  grid $itk_component(directory) -row 0 -column 1 -sticky ew -padx 10 -pady 0
  grid $itk_component(visDirectory) -row 1 -column 0 -columnspan 2 -sticky news -padx 10 -pady 0
  grid $itk_component(ff) -row 2 -column 0 -columnspan 2 -sticky ew -padx 5 -pady 5

  grid columnconfigure $itk_component(ff) 0 -weight 1
  grid $itk_component(selection) -sticky ew -row 0 -column 0 -columnspan 2 -pady 5 -padx 5
  grid $itk_component(filter)   -sticky ew -row 1 -column 0 -pady 5 -padx 5
  grid $itk_component(showHidden) -sticky ew -row 1 -column 1  -pady 5 -padx 5

  grid $itk_component(okButton) -sticky ew -row 0 -column 2 -pady 5 -padx 5
  grid $itk_component(cancelButton) -sticky ew -row 1 -column 2 -pady 5 -padx 5

  iwidgets::Labeledwidget::alignlabels \
    $itk_component(selection) \
    $itk_component(filter)

  # binding
  bind [$itk_component(filter) component entry] <Return> [code $this show]
  bind [$itk_component(selection) component entry] <Return> [code $this show]
    
  configure -modality application -master .

  eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## _okCommand
## ----------------------------------------------------------------------------
body FileSelector::_okCommand {} {

  set result ""
  set pwd [pwd]
  foreach item $selection {
    lappend result [file join $pwd $item]
  }

  # update the selection combobox
  set selection $result
  namespace inscope ::iwidgets::Combobox \
    [list $itk_component(selection) _addToList]

  deactivate
}

## ----------------------------------------------------------------------------
## _cancelCommand
## ----------------------------------------------------------------------------
body FileSelector::_cancelCommand {} {
  $itk_component(visDirectory) clearSelection
  
  deactivate
}

## ----------------------------------------------------------------------------
## _directoryCommand
## ----------------------------------------------------------------------------
body FileSelector::_directoryCommand {} {
  show $directory
}

## ----------------------------------------------------------------------------
## _homeCommand
## ----------------------------------------------------------------------------
body FileSelector::_homeCommand {} {
  show ~
}

## ----------------------------------------------------------------------------
## _chdirHandle
## ----------------------------------------------------------------------------
body FileSelector::_chdirHandle {} {

  set directory [pwd]

  # update upward button
  if {$directory == "/"} {
    $itk_component(toolbar) itemconfigure upward \
      -state disabled
  } else {
    $itk_component(toolbar) itemconfigure upward \
      -state normal
  }

  # update history 
  _updateHistory

  # update the filter component
  namespace inscope ::iwidgets::Combobox \
    [list $itk_component(filter) _addToList]

  # update the directory hierarchy
  $itk_component(directory) clear
  $itk_component(directory) selection clear 0 end
  set prefix ""
  foreach part [file split $directory] {
    set prefix [file join $prefix $part]
    $itk_component(directory) insert list 0 $prefix
  }
}

## ----------------------------------------------------------------------------
## _reloadCommand
## ----------------------------------------------------------------------------
body FileSelector::_reloadCommand {} {
  set _useHistory 0
  show
  set _useHistory 1
}

## ----------------------------------------------------------------------------
## _upCommand
## ----------------------------------------------------------------------------
body FileSelector::_upCommand {} {
  cd ..
  show [pwd]
}

## ----------------------------------------------------------------------------
## _updateHistory
## ----------------------------------------------------------------------------
body FileSelector::_updateHistory {} {
  if {!$_useHistory} {
    return
  }
  
  if {$directory == "."} {
    return
  }

  # truncate history
  set endIndex [expr [llength $_history] -1]
  if {$_historyIndex < $endIndex} {
    set _history [lreplace $_history [expr $_historyIndex +1] end]
    set endIndex $_historyIndex
  }

  # append history, omit double entries
  set lastDir [lindex $_history end]
  if {$lastDir != $directory} {
    lappend _history $directory
    incr endIndex
  }

  # set the current history index
  set _historyIndex [expr [llength $_history] -1]

  # update the balloons
  if {$_historyIndex > 0} {
    set prevDir [lindex $_history [expr $_historyIndex -1]]
    $itk_component(toolbar) itemconfigure backward \
      -balloonstr "back to $prevDir" \
      -state normal
  } else {
    $itk_component(toolbar) itemconfigure backward \
      -balloonstr "back" \
      -state disabled
  }
  $itk_component(toolbar) itemconfigure forward \
    -balloonstr "forward" \
    -state disabled

}

## ----------------------------------------------------------------------------
## _backwardHistory
## ----------------------------------------------------------------------------
body FileSelector::_backwardHistory {} {
  if {$_historyIndex == 0} {
    return
  }

  # show previous directory
  incr _historyIndex -1
  set _useHistory 0
  show [lindex $_history $_historyIndex]
  set _useHistory 1

  # udpdate the balloons of the bach and forward buttons
  if {$_historyIndex == 0} {
    $itk_component(toolbar) itemconfigure backward \
      -balloonstr "back" \
      -state disabled
  } else {
    set prevDir [lindex $_history [expr $_historyIndex -1]]
    $itk_component(toolbar) itemconfigure backward \
      -balloonstr "back to $prevDir" \
      -state normal
  }
  set nextDir [lindex $_history [expr $_historyIndex +1]]
  $itk_component(toolbar) itemconfigure forward \
    -balloonstr "forward to $nextDir" \
    -state normal

}

## ----------------------------------------------------------------------------
## _forwardHistory
## ----------------------------------------------------------------------------
body FileSelector::_forwardHistory {} {
  set endIndex [expr [llength $_history] -1]
  if {$_historyIndex >= $endIndex} {
    return
  }

  # show next directory
  incr _historyIndex
  set _useHistory 0
  show [lindex $_history $_historyIndex]
  set _useHistory 1

  # udpdate the balloons of the bach and forward buttons
  if {$_historyIndex == $endIndex} {
    $itk_component(toolbar) itemconfigure forward \
      -balloonstr "forward" \
      -state disabled
  } else {
    set nextDir [lindex $_history [expr $_historyIndex +1]]
    $itk_component(toolbar) itemconfigure forward \
      -balloonstr "forward to $nextDir" \
      -state normal
  }
  set prevDir [lindex $_history [expr $_historyIndex -1]]
  $itk_component(toolbar) itemconfigure backward \
      -balloonstr "back to $prevDir" \
      -state normal

}

## ----------------------------------------------------------------------------
## show
## ----------------------------------------------------------------------------
body FileSelector::show {{dirName ""}} {
  $itk_component(visDirectory) setFilter $_currentFilter
  $itk_component(visDirectory) setShowHidden $showHidden
  $itk_component(visDirectory) setMode $mode

  if {$dirName != ""} {
    set directory $dirName
  } 

  if {$directory == ""} {
    set directory [pwd]
  }

  $itk_component(visDirectory) show $directory
}

## ----------------------------------------------------------------------------
## _selectHandle
## ----------------------------------------------------------------------------
body FileSelector::_selectHandle {} {
  set selection [$itk_component(visDirectory) getSelection]
  if {$selection == ""} {
    $itk_component(toolbar) itemconfigure delete \
      -state disabled
  } else {
    $itk_component(toolbar) itemconfigure delete \
      -state normal
  }
}

## ----------------------------------------------------------------------------
## activate
## ----------------------------------------------------------------------------
body FileSelector::activate {args} {
  eval configure $args
  set _historyIndex 0
  set _history ""
  _selectionCommand
  center [cget -master]
  Shell::activate
  return "$selection"
}

## ----------------------------------------------------------------------------
## _selectionCommand
## ----------------------------------------------------------------------------
body FileSelector::_selectionCommand {} {
  set oldSelection "$selection"
  set selItem [lindex $selection 0]
  show [file dirname $selItem]
  $itk_component(visDirectory) select "$oldSelection"
}

## ----------------------------------------------------------------------------
## _newCommand
## ----------------------------------------------------------------------------
body FileSelector::_newCommand {} {
  $itk_component(visDirectory) new
}

## ----------------------------------------------------------------------------
## _deleteCommand
## ----------------------------------------------------------------------------
body FileSelector::_deleteCommand {} {
  if {$selection == ""} {
    return
  }

  $itk_component(message) configure -text \
    "Are you sure that you want to delete [llength $selection] file(s)?"

  $itk_component(message) center [$itk_component(message) cget -master]

  if {![$itk_component(message) activate]} {
    return
  } 
  
  eval file delete -- $selection

  show [pwd]
}

## ----------------------------------------------------------------------------
## -filter
## ----------------------------------------------------------------------------
configbody FileSelector::filter {
  $itk_component(filter) delete list 0 end
  $itk_component(filter) delete entry 0 end
  foreach item $itk_option(-filter) {
    set _currentFilter $item
    namespace inscope ::iwidgets::Combobox \
      [list $itk_component(filter) _addToList]
  }
  set _currentFilter [lindex $itk_option(-filter) 0]
}
