# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaRunner - document to run YadaJobs.
## This document is the basic implementation of a YadaRunnableDocument. A more
## specific implementation of a YadaRunnableDocument is for example the YadaGlsStats.
## We implement here the basic functionality to select an schedule YadaJobs
## that run on YadaMachines.
## \ingroup YadaRunnableDocuments
##
## \author Michael Daum 
##
## ----------------------------------------------------------------------------
class YadaRunner {
  inherit YadaRunnableDocument

  # private variables ----------------------------------------------------------------
  private variable _totalTime 0
  private variable _totalSolTime 0
  private variable _totalSoftTime 0
  private variable _totalScore 0
  private variable _noSolutions 0
  private variable _noErrors 0
  private variable _totalWordgraphs 0
  private variable _totalStructRecall 0
  private variable _totalLabelledRecall 0
  private variable _totalLexicalRecall 0
  private variable _totalStrictRecall 0
  private variable _typeOfColumn; ## \type TclArray
  private variable _scoreOf; ## \type TclArray
  private variable _structHitsOf; ## \type TclArray
  private variable _labelledHitsOf; ## \type TclArray
  private variable _lexicalHitsOf; ## \type TclArray
  private variable _strictHitsOf; ## \type TclArray
  private variable _triesOf; ## \type TclArray 
 

  # methods ------------------------------------------------------------------
  public method activationHandle {}
  public method clear {}
  public method toDom {}
  public method fromDom {document}; ## \type domDoc
  public method fromDomNode {runnerNode}; ## \type domNode
  public method getWindowTitle {}

  ## accessor methods
  public method getTotalTime {}
  public method getTotalSolTime {}
  public method getTotalSoftTime {}
  public method getTotalStructRecall {}
  public method getTotalLabelRecall {}
  public method getTotalLexRecall {}
  public method getTotalScore {}

  public method load {}; ## overrides implementation from YadaPlainDocument

  constructor {args} {}; ## \type TclList

  private method _analyseDocument {document experiment} ;## \type DomDocument, YadaExperiment
  private method _showStatistics {}
  private method _computeStatistics {}
  private method _judgeRow {row}; ## \type TclNumber
  private method _getTypeOfColumn {colNo} ;## \type TclNumber
  private method _resetDataOf {wordgraphs} ;## \type TclList
}

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaRunner::constructor {args} {
  # reconfigure table
  $itk_component(table) configure -rows 300 -cols 12 \
    -rowtagcommand [code $this _judgeRow]

  # set types of columns
  array set _typeOfColumn {
    0 "dictionary"
    1 "integer"
    2 "real"
    3 "integer"
    4 "integer"
    5 "integer"
    6 "integer"
    7 "real"
    8 "real"
    9 "real"
    10 "real"
  }


  itk_component add experimentBoxLabel {
    label $itk_component(heading).experimentBoxLabel \
	-text "Experiment:" \
	-anchor w
  }  {}

  itk_component add experimentBox {
    iwidgets::combobox $itk_component(heading).experimentBox \
	-completion false \
	-borderwidth 2 \
	-margin 4 \
	-selectioncommand [code $this selectExperiment]
  }  {}

  itk_component add totalScoreLabel {
    label $itk_component(heading).totalScoreLabel \
      -text "Score:" \
      -anchor w \
  } {}

  itk_component add totalScore {
    entry $itk_component(heading).totalScore \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalTimeLabel {
    label $itk_component(heading).totalTimeLabel \
      -text "Time:" \
      -anchor w 
  } {}

  itk_component add totalTime {
    entry $itk_component(heading).totalTime \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add noErrorsLabel {
    label $itk_component(heading).noErrorsLabel \
      -text "No.Errors:" \
      -anchor w 
  } {}

  itk_component add noErrors {
    entry $itk_component(heading).noErrors \
      -textvariable [scope _noErrors] \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add noSolutionsLabel {
    label $itk_component(heading).noSolutionsLabel \
      -text "No.Solutions:" \
      -anchor w 
  } {}

  itk_component add noSolutions {
    entry $itk_component(heading).noSolutions \
      -textvariable [scope _noSolutions] \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalSolTimeLabel {
    label $itk_component(heading).totalSolTimeLabel \
      -text "Sol.Time:" \
      -anchor w
  } {}

  itk_component add totalSolTime {
    entry $itk_component(heading).totalSolTime \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalSoftTimeLabel {
    label $itk_component(heading).totalSoftTimeLabel \
      -text "Soft.Time:" \
      -anchor w
  } {}

  itk_component add totalSoftTime {
    entry $itk_component(heading).totalSoftTime \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalStructRecallLabel {
    label $itk_component(heading).totalStructRecallLabel \
      -text "Struct.Recall:" \
      -anchor w
  } {}

  itk_component add totalStructRecall {
    entry $itk_component(heading).totalStructRecall \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalLabelledRecallLabel {
    label $itk_component(heading).totalLabelledRecallLabel \
      -text "Label.Recall:" \
      -anchor w
  } {}

  itk_component add totalLabelledRecall {
    entry $itk_component(heading).totalLabelledRecall \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalLexicalRecallLabel {
    label $itk_component(heading).totalLexicalRecallLabel \
      -text "Lex.Recall:" \
      -anchor w
  } {}

  itk_component add totalLexicalRecall {
    entry $itk_component(heading).totalLexicalRecall \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

  itk_component add totalStrictRecallLabel {
    label $itk_component(heading).totalStrictRecallLabel \
      -text "Strict.Recall:" \
      -anchor w
  } {}

  itk_component add totalStrictRecall {
    entry $itk_component(heading).totalStrictRecall \
      -borderwidth 2 \
      -state disabled \
      -width 20
  } {}

 # bindings
  set entry [$itk_component(experimentBox) component entry]
  bind $entry <Return> [code $this selectExperiment]
  bind $entry <Up> +[code $this selectExperiment]
  bind $entry <Down> +[code $this selectExperiment]

  # packing
  grid columnconfigure $itk_component(heading) 1 -weight 1
  grid columnconfigure $itk_component(heading) 2 -pad 5
  grid columnconfigure $itk_component(heading) 4 -weight 1
  grid columnconfigure $itk_component(heading) 5 -pad 5
  grid columnconfigure $itk_component(heading) 7 -weight 1
  grid columnconfigure $itk_component(heading) 8 -pad 5

  grid $itk_component(experimentBoxLabel)   -row 0 -column 0 -sticky ew -pady 2
  grid $itk_component(experimentBox)        -row 0 -column 1 -sticky ew -pady 2 -columnspan 4
  grid $itk_component(totalScoreLabel)  -row 1 -column 0 -sticky ew -pady 2
  grid $itk_component(totalScore)       -row 1 -column 1 -sticky ew -pady 2
  grid $itk_component(noErrorsLabel)    -row 2 -column 0 -sticky ew -pady 2
  grid $itk_component(noErrors)         -row 2 -column 1 -sticky ew -pady 2
  grid $itk_component(noSolutionsLabel) -row 3 -column 0 -sticky ew -pady 2
  grid $itk_component(noSolutions)      -row 3 -column 1 -sticky ew -pady 2

  grid $itk_component(totalTimeLabel)   -row 1 -column 3 -sticky ew -pady 2
  grid $itk_component(totalTime)        -row 1 -column 4 -sticky ew -pady 2
  grid $itk_component(totalSolTimeLabel) -row 2 -column 3 -sticky ew -pady 2
  grid $itk_component(totalSolTime)      -row 2 -column 4 -sticky ew -pady 2
  grid $itk_component(totalSoftTimeLabel) -row 3 -column 3 -sticky ew -pady 2
  grid $itk_component(totalSoftTime)      -row 3 -column 4 -sticky ew -pady 2

  grid $itk_component(totalStructRecallLabel) -row 0 -column 6 -sticky ew -pady 2
  grid $itk_component(totalStructRecall)      -row 0 -column 7 -sticky ew -pady 2
  grid $itk_component(totalLabelledRecallLabel) -row 1 -column 6 -sticky ew -pady 2
  grid $itk_component(totalLabelledRecall)      -row 1 -column 7 -sticky ew -pady 2
  grid $itk_component(totalLexicalRecallLabel)   -row 2 -column 6 -sticky ew -pady 2
  grid $itk_component(totalLexicalRecall)        -row 2 -column 7 -sticky ew -pady 2
  grid $itk_component(totalStrictRecallLabel) -row 3 -column 6 -sticky ew -pady 2
  grid $itk_component(totalStrictRecall)      -row 3 -column 7 -sticky ew -pady 2

  eval itk_initialize $args
  setModified
}

## ----------------------------------------------------------------------------
## activationHandle
## ----------------------------------------------------------------------------
body YadaRunner::activationHandle {} {
  chain ;# call YadaRunnableDocument::activationHandle

  # get the previous selection
  set box $itk_component(experimentBox)
  set experimentName [$box get]

  # default experiments to choose
  if {$experimentName == ""} {
    set experimentName "<none>"
  }

  set experimentNames [.main getActiveExperiments]

  set listOfEntries [[$box component list] get 0 end]
  if {$listOfEntries != $experimentNames} {
    $box clear list 
    eval $box insert list end $experimentNames
    selectExperiment $experimentName
  }

  if {$hasView && [lastChanged] != "" } {
    if { [lastChanged] > $_lastModificationTime } {
      if { [askNewLoad] } {
  	load
      } else {
  	setModified
      }
    }  
  }
}

## ----------------------------------------------------------------------------
## clear
## ----------------------------------------------------------------------------
body YadaRunner::clear {} {
  chain ;# call YadaRunnableDocument::clear

  # reset the table 
  set table $itk_component(table)
  $table setCell \
       	0,0 "wordgraph" \
       	0,1 "values" \
       	0,2 "score" \
	0,3 "errors" \
       	0,4 "time" \
       	0,5 "sol.time" \
       	0,6 "soft time" \
	0,7 "struct" \
	0,8 "labelled" \
	0,9 "lexical" \
	0,10 "strict" 


  $table tag config default -anchor e
  $table tag config error -bg red3 -fg white -anchor e -relief flat
  $table tag config colored -bg gray80 -anchor e

  $table tag config sel -fg "" -bg "" -relief raised -bd 1
  $table tag config active -anchor w -fg black -bg gray -relief sunk -bd 1
  $table tag config title -font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} -bg #a3a3a3 -fg white
  $table tag config runner -fg red3 -bg #a3a3a3
  $table tag raise runner title
  
  $table width 0 15 1 5 2 15 3 10 4 15 5 15 6 15 7 10 8 10 9 10 10 10

  _computeStatistics
}


## ----------------------------------------------------------------------------
## _analyseDocument
## ----------------------------------------------------------------------------
body YadaRunner::_analyseDocument {document experiment} {

  # default analysis
  YadaRunnableDocument::_analyseDocument $document $experiment

  # the wordgraph is needed at least
  if {$_analyseResult(wordgraph) == ""} {
    print "ERROR: no information available in this run\n"
    return
  }

  # save score and edge data
  set _scoreOf($_analyseResult(wordgraph)) $_analyseResult(score)
  set _structHitsOf($_analyseResult(wordgraph)) $_analyseResult(structHits)
  set _labelledHitsOf($_analyseResult(wordgraph)) $_analyseResult(labelledHits)
  set _lexicalHitsOf($_analyseResult(wordgraph)) $_analyseResult(lexicalHits)
  set _strictHitsOf($_analyseResult(wordgraph)) $_analyseResult(strictHits)
  set _triesOf($_analyseResult(wordgraph)) $_analyseResult(tries)

  if {$_analyseResult(tries) > 0} {
    set structRecall [expr $_analyseResult(structHits) * 100.0 / $_analyseResult(tries)] 
    set labelledRecall [expr $_analyseResult(labelledHits) * 100.0 / $_analyseResult(tries)] 
    set lexicalRecall [expr $_analyseResult(lexicalHits) * 100.0 / $_analyseResult(tries)]
    set strictRecall [expr $_analyseResult(strictHits) * 100.0 / $_analyseResult(tries)]   
  } else {
    set structRecall 0
    set labelledRecall 0
    set lexicalRecall 0
    set strictRecall 0
  }

  # add the collected data to the table
  _insertData \
    $_analyseResult(wordgraph) \
    $_analyseResult(noValues) \
    [format "%4.3e" $_analyseResult(score)] \
    [expr $_analyseResult(tries) - $_analyseResult(lexicalHits)] \
    $_analyseResult(totalTime) \
    $_analyseResult(solutionTime) \
    $_analyseResult(softTime) \
    [format "%3.2f" $structRecall ] \
    [format "%3.2f" $labelledRecall ] \
    [format "%3.2f" $lexicalRecall ] \
    [format "%3.2f" $strictRecall ]

  setModified
}

## ----------------------------------------------------------------------------
## _computeStatistics
## ----------------------------------------------------------------------------
body YadaRunner::_computeStatistics {} {
  chain

  set table $itk_component(table)
  set rmax [$table cget -rows]
  set cmax [$table cget -cols]

  set _totalWordgraphs 0
  set _totalScore 0
  set _noSolutions 0
  set _noErrors 0
  set _totalSolTime 0
  set _totalSoftTime 0
  set _totalTime 0
  set _totalStructRecall 0
  set _totalLabelledRecall 0
  set _totalLexicalRecall 0
  set _totalStrictRecall 0

  set totalTries 0
  set totalStructHits 0
  set totalLabelledHits 0
  set totalLexicalHits 0
  set totalStrictHits 0

  for {set row 1} {$row < $rmax} {incr row} {

    set dataRow [$table getCell $row,0 $row,$cmax]
    set noValues	[lindex $dataRow 1]
    if {$noValues == ""} {
      continue
    }
 
    set wordgraph	[lindex $dataRow 0]
    set errors		[lindex $dataRow 3]
    set time		[lindex $dataRow 4]
    set solTime		[lindex $dataRow 5]
    set softTime	[lindex $dataRow 6]

    incr _totalWordgraphs
    incr _totalTime $time
    incr _totalSolTime $solTime
    incr _totalSoftTime $softTime
    incr _noErrors $errors

    set _totalScore [expr $_totalScore + $_scoreOf($wordgraph)]
    if {$_scoreOf($wordgraph) > 0.0} {
      incr _noSolutions
    }

    incr totalTries $_triesOf($wordgraph)
    incr totalStructHits $_structHitsOf($wordgraph)
    incr totalLabelledHits $_labelledHitsOf($wordgraph)
    incr totalLexicalHits $_lexicalHitsOf($wordgraph)
    incr totalStrictHits $_strictHitsOf($wordgraph)
  }

  if {$totalTries > 0} {
    set _totalStructRecall [expr $totalStructHits * 100.0 / $totalTries ]
    set _totalLabelledRecall [expr $totalLabelledHits * 100.0 / $totalTries ]
    set _totalLexicalRecall [expr $totalLexicalHits * 100.0 / $totalTries ]
    set _totalStrictRecall [expr $totalStrictHits * 100.0 / $totalTries ]
  }

  _showStatistics
}

## ----------------------------------------------------------------------------
## _showStatistics
## ----------------------------------------------------------------------------
body YadaRunner::_showStatistics {} {
  $itk_component(totalTime) configure -state normal
  $itk_component(totalSoftTime) configure -state normal
  $itk_component(totalSolTime) configure -state normal
  $itk_component(totalScore) configure -state normal
  $itk_component(totalStructRecall) configure -state normal
  $itk_component(totalLabelledRecall) configure -state normal
  $itk_component(totalLexicalRecall) configure -state normal
  $itk_component(totalStrictRecall) configure -state normal

  $itk_component(totalTime) delete 0 end
  $itk_component(totalSolTime) delete 0 end
  $itk_component(totalSoftTime) delete 0 end
  $itk_component(totalScore) delete 0 end
  $itk_component(totalStructRecall) delete 0 end
  $itk_component(totalLabelledRecall) delete 0 end
  $itk_component(totalLexicalRecall) delete 0 end
  $itk_component(totalStrictRecall) delete 0 end

  $itk_component(totalScore) insert 0 [format "%.4f" $_totalScore]
  $itk_component(totalTime) insert 0 "[timeToString $_totalTime]"
  $itk_component(totalSolTime) insert 0 "[timeToString $_totalSolTime]"
  $itk_component(totalSoftTime) insert 0 "[timeToString $_totalSoftTime]"
  $itk_component(totalStructRecall) insert 0 [format "%.4f%%" $_totalStructRecall]
  $itk_component(totalLabelledRecall) insert 0 [format "%.4f%%" $_totalLabelledRecall]
  $itk_component(totalLexicalRecall) insert 0 [format "%.4f%%" $_totalLexicalRecall]
  $itk_component(totalStrictRecall) insert 0 [format "%.4f%%" $_totalStrictRecall]

  $itk_component(totalTime) configure -state disabled
  $itk_component(totalSolTime) configure -state disabled
  $itk_component(totalSoftTime) configure -state disabled
  $itk_component(totalScore) configure -state disabled
  $itk_component(totalStructRecall) configure -state disabled
  $itk_component(totalLabelledRecall) configure -state disabled
  $itk_component(totalLexicalRecall) configure -state disabled
  $itk_component(totalStrictRecall) configure -state disabled
}

## ----------------------------------------------------------------------------
## _judgeRow
## ----------------------------------------------------------------------------
body YadaRunner::_judgeRow {row} {
  set errors [$itk_component(table) getCell $row,3]
  set score  [$itk_component(table) getCell $row,2]
  if {$errors > 0 || $score == 0.0} {
    return "error"
  } else {
    if {[expr $row % 2]} {
      return "default"
    } else {
      return "colored"
    }
  }
}

## ----------------------------------------------------------------------------
## _getTypeOfColumn
## ----------------------------------------------------------------------------
body YadaRunner::_getTypeOfColumn {colNo} {
  return $_typeOfColumn($colNo)
}

## ----------------------------------------------------------------------------
## _resetDataOf
## ----------------------------------------------------------------------------
body YadaRunner::_resetDataOf {wordgraphs} {
  chain $wordgraphs
  foreach wordgraph $wordgraphs {
    set _scoreOf($wordgraph) 0.0
    set _triesOf($wordgraph) 0
    set _structHitsOf($wordgraph) 0
    set _labelledHitsOf($wordgraph) 0
    set _lexicalHitsOf($wordgraph) 0
    set _strictHitsOf($wordgraph) 0
  }
}

## ----------------------------------------------------------------------------
## serializes this runmner into a dom document
## @returns a new dom document
## ----------------------------------------------------------------------------
body YadaRunner::toDom {} {

  set document [dom createDocument "yada"]
  set root [$document documentElement]

  set experimentName [$itk_component(experimentBox) get]

  set runnerNode [$document createElement "runner"]
  $root appendChild $runnerNode
  $runnerNode setAttribute "name" $experimentName 

  set experimentNode [$document createElement "experimentRef"]
  $runnerNode appendChild $experimentNode
  $experimentNode setAttribute "name" "$experimentName"

#  $runnerNode setAttribute "xmlns:xlink" "http://www.w3.org/1999/xlink"
#  $runnerNode setAttribute "xlink:type" "simple"
#  $runnerNode setAttribute "xlink:href"  [$experiment getFileName]

  set statsNode [$document createElement "stats"]
  $runnerNode appendChild $statsNode
  $statsNode setAttribute "score" $_totalScore
  $statsNode setAttribute "numErrors" $_noErrors
  $statsNode setAttribute "numSolutions" $_noSolutions
  $statsNode setAttribute "time" $_totalTime
  $statsNode setAttribute "solTime" $_totalSolTime
  $statsNode setAttribute "softTime" $_totalSoftTime
  $statsNode setAttribute "structRecall" $_totalStructRecall
  $statsNode setAttribute "labelRecall" $_totalLabelledRecall
  $statsNode setAttribute "lexicalRecall" $_totalLexicalRecall
  $statsNode setAttribute "strictRecall" $_totalStrictRecall


  set table $itk_component(table)
  set rmax [$table cget -rows]
  set cmax [expr [$table cget -cols] -2]
  for {set row 1} {$row < $rmax} {incr row} {
    set dataRow [$table getCell $row,0 $row,$cmax]
    if {[lindex $dataRow 2] == ""} {
        continue
    }

    set wordgraphName [lindex $dataRow 0]
    set wordgraphNode [$document createElement "wordgraph"]
    $runnerNode appendChild $wordgraphNode
    $wordgraphNode setAttribute "name" $wordgraphName
    $wordgraphNode setAttribute "values" [lindex $dataRow 1]
    $wordgraphNode setAttribute "score" [lindex $dataRow 2]
    $wordgraphNode setAttribute "errors" [lindex $dataRow 3]
    $wordgraphNode setAttribute "time" [lindex $dataRow 4]
    $wordgraphNode setAttribute "solTime" [lindex $dataRow 5]
    $wordgraphNode setAttribute "softTime" [lindex $dataRow 6]
    $wordgraphNode setAttribute "structRecall" [lindex $dataRow 7]
    $wordgraphNode setAttribute "labelRecall" [lindex $dataRow 8]
    $wordgraphNode setAttribute "lexicalRecall" [lindex $dataRow 9]
    $wordgraphNode setAttribute "strictRecall" [lindex $dataRow 10]
    $wordgraphNode setAttribute "structHits" $_structHitsOf($wordgraphName)
    $wordgraphNode setAttribute "labelledHits" $_labelledHitsOf($wordgraphName)
    $wordgraphNode setAttribute "lexicalHits" $_lexicalHitsOf($wordgraphName)
    $wordgraphNode setAttribute "strictHits" $_strictHitsOf($wordgraphName)
    $wordgraphNode setAttribute "tries" $_triesOf($wordgraphName)
  }

  return $document
}

## ----------------------------------------------------------------------------
## initialize this runner from a dom node
## @param document containing a /yada/runner
## ----------------------------------------------------------------------------
body YadaRunner::fromDom {document} {
  set rootNode [$document documentElement]
  set runnerNode [$rootNode selectNodes "/yada/runner"]
  fromDomNode $runnerNode
}

## ----------------------------------------------------------------------------
## initialize this runner from a dom node
## @param runnerNode containing a runner description
## \TODO this refreshes the display also -> should be a separate step
## ----------------------------------------------------------------------------
body YadaRunner::fromDomNode {runnerNode} {

  set experiment [getSelectedExperiment]
  if {$experiment == ""} {
    return
  }

  # get the timestamp for this document
  file stat [getZippedFileName] statDocumentFile

  # get set of wordgraphs to be loaded
  set activeWordgraphs [_getActiveWordgraphs]
  if {$activeWordgraphs == ""} {
    clear
    set activeWordgraphs [_getWordgraphs]
  }

  # read wordgraph from the document's xml file
  set updatedWordgraphs {}
  set oldFollowDataFlag $_followDataFlag
  set _followDataFlag 0
  foreach wordgraph $activeWordgraphs {
    print "INFO: loading $wordgraph\n"

    # get description
    set node [$runnerNode selectNodes "./wordgraph\[@name=\"$wordgraph\"\]" 0]

    # if there's none then load this wordgraph from its own file
    if {$node == ""} {
      # remember this node to be loaded
      lappend updatedWordgraphs $wordgraph
      continue
    }

    # only load wordgraphs from the document file, 
    # if there is no updated wordgraph xml file
    set wordgraphFileName [$experiment getXmlFileName $wordgraph]
    if {![catch {file stat $wordgraphFileName statWordgraphFile}]} {
      if {[expr {$statDocumentFile(ctime) < $statWordgraphFile(ctime)}]} {
	lappend updatedWordgraphs $wordgraph
	continue
      }
    }

    set values [$node getAttribute "values"]
    set score [$node getAttribute "score"]
    set errors [$node getAttribute "errors"]
    set time [$node getAttribute "time"]
    set solTime [$node getAttribute "solTime"]
    set softTime [$node getAttribute "softTime"]
    set structRecall [$node getAttribute "structRecall"]
    set labelRecall [$node getAttribute "labelRecall"]
    set lexicalRecall [$node getAttribute "lexicalRecall"]
    set strictRecall [$node getAttribute "strictRecall"]

    set _scoreOf($wordgraph) $score
    set _structHitsOf($wordgraph) [$node getAttribute "structHits"]
    set _labelledHitsOf($wordgraph) [$node getAttribute "labelledHits"]
    set _lexicalHitsOf($wordgraph) [$node getAttribute "lexicalHits"]
    set _strictHitsOf($wordgraph) [$node getAttribute "strictHits"]
    set _triesOf($wordgraph) [$node getAttribute "tries"]

    _insertData $wordgraph\
        $values\
        $score\
        $errors\
        $time\
        $solTime\
        $softTime\
        $structRecall\
        $labelRecall\
        $lexicalRecall\
        $strictRecall
   
    update
  }
  if {$updatedWordgraphs != ""} {
    loadWordgraphs $updatedWordgraphs
  }
  _computeStatistics
  set _followDataFlag $oldFollowDataFlag
}


## ----------------------------------------------------------------------------
## get the title string representing the state of this document
## ----------------------------------------------------------------------------
body YadaRunner::getWindowTitle {} {

  set experiment [getSelectedExperiment]
  if {$experiment == "" || $experiment == "<none>"} {
    return [chain]
  } 

  set experimentName [$experiment cget -name]
  return "[chain]: $experimentName"
}

## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalTime {} {
    return $_totalTime
}

## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalSolTime {} {
    return $_totalSolTime
}

## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalSoftTime {} {
    return $_totalSoftTime
}

## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalStructRecall {} {
    return $_totalStructRecall
}


## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalLabelRecall {} {
    return $_totalLabelledRecall
}


## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalLexRecall {} {
    return $_totalLexicalRecall
}

## ----------------------------------------------------------------------------
## Returns value of private attribute
## ----------------------------------------------------------------------------
body YadaRunner::getTotalScore {} {
    return $_totalScore
}

## ----------------------------------------------------------------------------
## Overridden to handle missing file to load from
## ----------------------------------------------------------------------------
body YadaRunner::load {} {
  if {![chain]} {
    loadWordgraphs
    _computeStatistics
    setModified
  } else {
    print "INFO: loaded '[getZippedFileName]'\n"
  }
}
