import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

import junit.framework.TestCase;

public class SentencesParserTest extends TestCase {

    SentencesParser _parser;

    @Override
    protected void setUp() throws Exception {
	_parser = new SentencesParser(
		"/home/j2005/5hinrich/cdg2/grammar/deutsch/deutsch-lexikon.cdg");
    }

    public void testParseWordgraphForGenitiveReading() throws IOException {
	BufferedReader in = new BufferedReader(new InputStreamReader(
		new FileInputStream("data/011.txt"), Charset
			.forName("ISO-8859-1")));
	ThematicRelations relations = _parser
		.parseWordgraphForGenitiveReading(in);
	in.close();
	assertEquals("011", relations.getID());
	// Only the words 6, 8 and 10 should be dependants of thematic
	// relations:
	for (int i = 1; i <= 11; ++i) {
	    if (i != 6 && i != 8 && i != 10) {
		assertFalse("Superfluous thematic relation for word: " + i,
			relations.hasRole(i));
	    }
	}
	// word 6 should be AGENT of word 11:
	assertEquals("Magd", relations.getWord(6));
	assertTrue(relations.hasBaseConcept(6));
	assertSame(BaseConcept.PERSON, relations.getBaseConcept(6));
	assertTrue(relations.hasRole(6));
	assertSame(Role.AGENT, relations.getRole(6));
	assertSame(11, relations.getRegent(6));
	assertTrue(relations.getIndividuals().contains(6));
	// word 8 should be OWNER of word 6:
	assertEquals("Baeuerin", relations.getWord(8));
	assertTrue(relations.hasBaseConcept(8));
	assertSame(BaseConcept.PERSON, relations.getBaseConcept(8));
	assertTrue(relations.hasRole(8));
	assertSame(Role.OWNER, relations.getRole(8));
	assertSame(6, relations.getRegent(8));
	assertTrue(relations.getIndividuals().contains(8));
	// word 10 should be THEME of word 11:
	assertEquals("Korb", relations.getWord(10));
	assertTrue(relations.hasBaseConcept(10));
	assertSame(BaseConcept.OBJECT, relations.getBaseConcept(10));
	assertTrue(relations.hasRole(10));
	assertSame(Role.THEME, relations.getRole(10));
	assertSame(11, relations.getRegent(10));
	assertTrue(relations.getIndividuals().contains(10));
	// word 11 should have semantic value ag_th:
	assertEquals("suchen", relations.getWord(11));
	assertFalse(relations.hasBaseConcept(11));
	assertTrue(relations.hasSemVal(11));
	assertSame(SemVal.ag_th, relations.getSemVal(11));
	assertTrue(relations.getIndividuals().contains(11));
	// This should be a readable reconstruction of the original sentence:
	assertEquals("Er wusste, dass die Magd der Buerin den Korb suchte.",
		relations.getSentence());
    }

    public void testParseWordgraphForDativeReading() throws IOException {
	BufferedReader in = new BufferedReader(new InputStreamReader(
		new FileInputStream("data/011.txt"), Charset
			.forName("ISO-8859-1")));
	ThematicRelations relations = _parser
		.parseWordgraphForDativeReading(in);
	in.close();
	assertEquals("011", relations.getID());
	// Only the words 6, 8 and 10 should be dependants of thematic
	// relations:
	for (int i = 1; i <= 11; ++i) {
	    if (i != 6 && i != 8 && i != 10) {
		assertFalse("Superfluous thematic relation for word: " + i,
			relations.hasRole(i));
	    }
	}
	// word 6 should be AGENT of word 11:
	assertEquals("Magd", relations.getWord(6));
	assertTrue(relations.hasBaseConcept(6));
	assertSame(BaseConcept.PERSON, relations.getBaseConcept(6));
	assertTrue(relations.hasRole(6));
	assertSame(Role.AGENT, relations.getRole(6));
	assertSame(11, relations.getRegent(6));
	assertTrue(relations.getIndividuals().contains(6));
	// word 8 should be RECIPIENT of word 11:
	assertEquals("Baeuerin", relations.getWord(8));
	assertTrue(relations.hasBaseConcept(8));
	assertSame(BaseConcept.PERSON, relations.getBaseConcept(8));
	assertTrue(relations.hasRole(8));
	assertSame(Role.RECIPIENT, relations.getRole(8));
	assertSame(11, relations.getRegent(8));
	assertTrue(relations.getIndividuals().contains(8));
	// word 10 should be THEME of word 11:
	assertEquals("Korb", relations.getWord(10));
	assertTrue(relations.hasBaseConcept(10));
	assertSame(BaseConcept.OBJECT, relations.getBaseConcept(10));
	assertTrue(relations.hasRole(10));
	assertSame(Role.THEME, relations.getRole(10));
	assertSame(11, relations.getRegent(10));
	assertTrue(relations.getIndividuals().contains(10));
	// word 11 should have semantic value ag_re_th:
	assertEquals("suchen", relations.getWord(11));
	assertFalse(relations.hasBaseConcept(11));
	assertTrue(relations.hasSemVal(11));
	assertSame(SemVal.ag_re_th, relations.getSemVal(11));
	assertTrue(relations.getIndividuals().contains(11));
	// This should be a readable reconstruction of the original sentence:
	assertEquals("Er wusste, dass die Magd der Buerin den Korb suchte.",
		relations.getSentence());
    }
}
